@@ -119,7 +119,7 @@ time_t Time_now(void);
  * Coordinated Universal Time (UTC).
  * @exception AssertException If time could not be obtained
  */
-long long int Time_milli(void);
+long long Time_milli(void);
 
 
 /**
@@ -129,7 +129,7 @@ long long int Time_milli(void);
  * Coordinated Universal Time (UTC).
  * @exception AssertException If time could not be obtained
  */
-long long int Time_micro(void);
+long long Time_micro(void);
 
 
 /**
@@ -235,7 +235,7 @@ char *Time_fmt(char *result, int size, const char *format, time_t time);
 
 
 /**
- * Returns a uptime formated string for the given seconds. That is, convert
+ * Returns a uptime formatted string for the given seconds. That is, convert
  * <code>sec</code> to days, hours and minutes and return a string on the
  * form, <code>7d, 17h, 34m</code>. The submitted result buffer must be
  * large enough to hold at least 24 bytes.
