@@ -54,14 +54,12 @@
 
 #include "monit.h"
 #include "ProcessTree.h"
-#include "net.h"
-#include "socket.h"
 #include "event.h"
 #include "util.h"
 #include "system/Time.h"
 
 // libmonit
-#include "util/Fmt.h"
+#include "util/Convert.h"
 #include "exceptions/AssertException.h"
 
 
@@ -93,7 +91,7 @@ static int _getOutput(InputStream_T in, char *buf, int buflen) {
 }
 
 
-static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int64_t *timeout) {
+static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, long long *timeout) {
         ASSERT(S);
         ASSERT(c);
         ASSERT(msg);
@@ -111,7 +109,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
         }
         END_TRY;
         if (C) {
-                int64_t _timeoutMilli = *timeout / 1000.;
+                long long _timeoutMilli = *timeout / 1000.;
                 for (int i = 1; i < c->length; i++)
                         Command_appendArgument(C, c->arg[i]);
                 if (c->has_uid)
@@ -144,7 +142,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
                                 *timeout -= RETRY_INTERVAL;
                         } while ((status = Process_exitStatus(P)) < 0 && *timeout > 0 && ! (Run.flags & Run_Stopped));
                         if (*timeout <= 0)
-                                snprintf(msg, msglen, "Program '%s' timed out after %s", Util_commandDescription(c, (char[STRLEN]){}), Fmt_time2str(_timeoutMilli, (char[11]){}));
+                                snprintf(msg, msglen, "Program '%s' timed out after %s", Util_commandDescription(c, (char[STRLEN]){}), Convert_time2str(_timeoutMilli, (char[11]){}));
                         int n, total = 0;
                         char buf[STRLEN];
                         do {
@@ -166,7 +164,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
 }
 
 
-static Process_Status _waitProcessStart(Service_T s, int64_t *timeout) {
+static Process_Status _waitProcessStart(Service_T s, long long *timeout) {
         long wait = RETRY_INTERVAL;
         do {
                 Time_usleep(wait);
@@ -183,7 +181,7 @@ static Process_Status _waitProcessStart(Service_T s, int64_t *timeout) {
 }
 
 
-static Process_Status _waitProcessStop(int pid, int64_t *timeout) {
+static Process_Status _waitProcessStop(int pid, long long *timeout) {
         do {
                 Time_usleep(RETRY_INTERVAL);
                 if (! pid || (getpgid(pid) == -1 && errno != EPERM))
@@ -203,7 +201,7 @@ static State_Type _check(Service_T s) {
         rv = s->check(s);
         if (s->type == Service_Program && s->program->P) {
                 // check program executes the program and needs to be called again to collect the exit value and evaluate the status
-                int64_t timeout = s->program->timeout * USEC_PER_MSEC;
+                long long timeout = s->program->timeout * USEC_PER_MSEC;
                 do {
                         Time_usleep(RETRY_INTERVAL);
                         timeout -= RETRY_INTERVAL;
@@ -221,9 +219,9 @@ static State_Type _check(Service_T s) {
  * @param s A Service_T object
  * @return true if the service was started otherwise false
  */
-static boolean_t _doStart(Service_T s) {
+static bool _doStart(Service_T s) {
         ASSERT(s);
-        boolean_t rv = true;
+        bool rv = true;
         StringBuffer_T sb = StringBuffer_create(64);
         for (Dependant_T d = s->dependantlist; d; d = d->next ) {
                 Service_T parent = Util_getService(d->dependant);
@@ -243,7 +241,7 @@ static boolean_t _doStart(Service_T s) {
                         if (s->type != Service_Process || ! ProcessTree_findProcess(s)) {
                                 LogInfo("'%s' start: '%s'\n", s->name, Util_commandDescription(s->start, (char[STRLEN]){}));
                                 char msg[1024];
-                                int64_t timeout = s->start->timeout * USEC_PER_MSEC;
+                                long long timeout = s->start->timeout * USEC_PER_MSEC;
                                 int status = _commandExecute(s, s->start, msg, sizeof(msg), &timeout);
                                 if (status < 0 || (s->type == Service_Process && _waitProcessStart(s, &timeout) != Process_Started)) {
                                         Event_post(s, Event_Exec, State_Failed, s->action_EXEC, "failed to start (exit status %d) -- %s", status, *msg ? msg : "no output");
@@ -266,13 +264,13 @@ static boolean_t _doStart(Service_T s) {
 }
 
 
-static int _executeStop(Service_T s, char *msg, int msglen, int64_t *timeout) {
+static int _executeStop(Service_T s, char *msg, int msglen, long long *timeout) {
         LogInfo("'%s' stop: '%s'\n", s->name, Util_commandDescription(s->stop, (char[STRLEN]){}));
         return _commandExecute(s, s->stop, msg, msglen, timeout);
 }
 
 
-static void _evaluateStop(Service_T s, boolean_t succeeded, int exitStatus, char *msg) {
+static void _evaluateStop(Service_T s, bool succeeded, int exitStatus, char *msg) {
         if (succeeded)
                 Event_post(s, Event_Exec, State_Succeeded, s->action_EXEC, "stopped");
         else
@@ -286,14 +284,14 @@ static void _evaluateStop(Service_T s, boolean_t succeeded, int exitStatus, char
  * @param unmonitor true if the monitoring should be disabled or false if monitoring should continue (when stop is part of restart)
  * @return true if the service was stopped otherwise false
  */
-static boolean_t _doStop(Service_T s, boolean_t unmonitor) {
+static bool _doStop(Service_T s, bool unmonitor) {
         ASSERT(s);
-        boolean_t rv = true;
+        bool rv = true;
         if (s->stop) {
                 if (s->monitor != Monitor_Not) {
                         int exitStatus;
                         char msg[1024];
-                        int64_t timeout = s->stop->timeout * USEC_PER_MSEC;
+                        long long timeout = s->stop->timeout * USEC_PER_MSEC;
                         if (s->type == Service_Process) {
                                 int pid = ProcessTree_findProcess(s);
                                 if (pid) {
@@ -324,14 +322,14 @@ static boolean_t _doStop(Service_T s, boolean_t unmonitor) {
  * This function simply restarts the service s.
  * @param s A Service_T object
  */
-static boolean_t _doRestart(Service_T s) {
+static bool _doRestart(Service_T s) {
         ASSERT(s);
-        boolean_t rv = true;
+        bool rv = true;
         if (s->restart) {
                 LogInfo("'%s' restart: '%s'\n", s->name, Util_commandDescription(s->restart, (char[STRLEN]){}));
                 Util_resetInfo(s);
                 char msg[1024];
-                int64_t timeout = s->restart->timeout * USEC_PER_MSEC;
+                long long timeout = s->restart->timeout * USEC_PER_MSEC;
                 int status = _commandExecute(s, s->restart, msg, sizeof(msg), &timeout);
                 if (status < 0 || (s->type == Service_Process && _waitProcessStart(s, &timeout) != Process_Started)) {
                         rv = false;
@@ -380,9 +378,9 @@ static void _doUnmonitor(Service_T s) {
  * @param unmonitor Disable service monitoring: used for stop action only to differentiate hard/soft stop - see _doStop()
  * @return true if all depending services were started/stopped otherwise false
  */
-static boolean_t _doDepend(Service_T s, Action_Type action, boolean_t unmonitor) {
+static bool _doDepend(Service_T s, Action_Type action, bool unmonitor) {
         ASSERT(s);
-        boolean_t rv = true;
+        bool rv = true;
         for (Service_T child = servicelist; child; child = child->next) {
                 for (Dependant_T d = child->dependantlist; d; d = d->next) {
                         if (IS(d->dependant, s->name)) {
@@ -429,7 +427,7 @@ static boolean_t _doDepend(Service_T s, Action_Type action, boolean_t unmonitor)
  * @param action A string describing the action to execute
  * @return number of errors
  */
-boolean_t control_service_string(List_T services, const char *action) {
+bool control_service_string(List_T services, const char *action) {
         ASSERT(services);
         ASSERT(action);
         Action_Type a = Util_getAction(action);
@@ -451,9 +449,9 @@ boolean_t control_service_string(List_T services, const char *action) {
  * @param A An action id describing the action to execute
  * @return false for error, otherwise true
  */
-boolean_t control_service(const char *S, Action_Type A) {
+bool control_service(const char *S, Action_Type A) {
         Service_T s = NULL;
-        boolean_t rv = true;
+        bool rv = true;
         ASSERT(S);
         if (! (s = Util_getService(S))) {
                 LogError("Service '%s' -- doesn't exist\n", S);
