@@ -75,6 +75,7 @@
 #endif
 
 #include "monit.h"
+#include "device.h"
 
 // libmonit
 #include "system/Time.h"
@@ -85,7 +86,7 @@
 
 
 static struct {
-        uint64_t timestamp;
+        unsigned long long timestamp;
         struct statinfo disk;
 } _statistics = {};
 
@@ -106,13 +107,13 @@ static void __attribute__ ((destructor)) _destructor() {
 /* ----------------------------------------------------------------- Private */
 
 
-static uint64_t _timevalToMilli(struct timeval *time) {
+static unsigned long long _timevalToMilli(struct timeval *time) {
         return time->tv_sec * 1000 + time->tv_usec / 1000.;
 }
 
 
 // Parse the device path like /dev/da0p2 into name:instance -> da:0
-static boolean_t _parseDevice(const char *path, Device_T device) {
+static bool _parseDevice(const char *path, Device_T device) {
         const char *base = File_basename(path);
         for (int i = 0; base[i]; i++) {
                 if (isdigit(*(base + i))) {
@@ -126,7 +127,7 @@ static boolean_t _parseDevice(const char *path, Device_T device) {
 }
 
 
-static boolean_t _getStatistics(uint64_t now) {
+static bool _getStatistics(unsigned long long now) {
         // Refresh only if the statistics are older then 1 second (handle also backward time jumps)
         if (now > _statistics.timestamp + 1000 || now < _statistics.timestamp - 1000) {
                 if (getdevs(&(_statistics.disk)) == -1) {
@@ -139,19 +140,19 @@ static boolean_t _getStatistics(uint64_t now) {
 }
 
 
-static boolean_t _getDummyDiskActivity(void *_inf) {
+static bool _getDummyDiskActivity(void *_inf) {
         return true;
 }
 
 
-static boolean_t _getBlockDiskActivity(void *_inf) {
+static bool _getBlockDiskActivity(void *_inf) {
         Info_T inf = _inf;
-        uint64_t now = Time_milli();
-        boolean_t rv = _getStatistics(now);
+        unsigned long long now = Time_milli();
+        bool rv = _getStatistics(now);
         if (rv) {
                 for (int i = 0; i < _statistics.disk.dinfo->numdevs; i++) {
                         if (_statistics.disk.dinfo->devices[i].unit_number == inf->filesystem->object.instance && IS(_statistics.disk.dinfo->devices[i].device_name, inf->filesystem->object.key)) {
-                                uint64_t now = Time_milli();
+                                unsigned long long now = Time_milli();
                                 Statistics_update(&(inf->filesystem->read.bytes), now, _statistics.disk.dinfo->devices[i].bytes_read);
                                 Statistics_update(&(inf->filesystem->read.operations),  now, _statistics.disk.dinfo->devices[i].num_reads);
                                 Statistics_update(&(inf->filesystem->write.bytes), now, _statistics.disk.dinfo->devices[i].bytes_written);
@@ -165,7 +166,7 @@ static boolean_t _getBlockDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _getDiskUsage(void *_inf) {
+static bool _getDiskUsage(void *_inf) {
         Info_T inf = _inf;
         struct statfs usage;
         if (statfs(inf->filesystem->object.mountpoint, &usage) != 0) {
@@ -182,19 +183,19 @@ static boolean_t _getDiskUsage(void *_inf) {
 }
 
 
-static boolean_t _compareMountpoint(const char *mountpoint, struct statfs *mnt) {
+static bool _compareMountpoint(const char *mountpoint, struct statfs *mnt) {
         return IS(mountpoint, mnt->f_mntonname);
 }
 
 
-static boolean_t _compareDevice(const char *device, struct statfs *mnt) {
+static bool _compareDevice(const char *device, struct statfs *mnt) {
         return IS(device, mnt->f_mntfromname);
 }
 
 
-static void _filesystemFlagsToString(Info_T inf, uint64_t flags) {
+static void _filesystemFlagsToString(Info_T inf, unsigned long long flags) {
         struct mystable {
-                uint64_t flag;
+                unsigned long long flag;
                 char *description;
         } t[]= {
                 {MNT_ASYNC, "async"},
@@ -232,7 +233,7 @@ static void _filesystemFlagsToString(Info_T inf, uint64_t flags) {
 }
 
 
-static boolean_t _setDevice(Info_T inf, const char *path, boolean_t (*compare)(const char *path, struct statfs *mnt)) {
+static bool _setDevice(Info_T inf, const char *path, bool (*compare)(const char *path, struct statfs *mnt)) {
         int countfs = getfsstat(NULL, 0, MNT_NOWAIT);
         if (countfs != -1) {
                 struct statfs *mnt = CALLOC(countfs, sizeof(struct statfs));
@@ -277,7 +278,7 @@ error:
 }
 
 
-static boolean_t _getDevice(Info_T inf, const char *path, boolean_t (*compare)(const char *path, struct statfs *mnt)) {
+static bool _getDevice(Info_T inf, const char *path, bool (*compare)(const char *path, struct statfs *mnt)) {
         if (_setDevice(inf, path, compare)) {
                 return (inf->filesystem->object.getDiskUsage(inf) && inf->filesystem->object.getDiskActivity(inf));
         }
@@ -288,14 +289,14 @@ static boolean_t _getDevice(Info_T inf, const char *path, boolean_t (*compare)(c
 /* ------------------------------------------------------------------ Public */
 
 
-boolean_t Filesystem_getByMountpoint(Info_T inf, const char *path) {
+bool Filesystem_getByMountpoint(Info_T inf, const char *path) {
         ASSERT(inf);
         ASSERT(path);
         return _getDevice(inf, path, _compareMountpoint);
 }
 
 
-boolean_t Filesystem_getByDevice(Info_T inf, const char *path) {
+bool Filesystem_getByDevice(Info_T inf, const char *path) {
         ASSERT(inf);
         ASSERT(path);
         return _getDevice(inf, path, _compareDevice);
