@@ -70,7 +70,12 @@
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_SYS_SYSMACROS_H
+#include <sys/sysmacros.h>
+#endif
+
 #include "monit.h"
+#include "device.h"
 
 // libmonit
 #include "io/File.h"
@@ -90,15 +95,15 @@
 static struct {
         int fd;                                    // /proc/self/mounts filedescriptor (needed for mount/unmount notification)
         int generation;                            // Increment each time the mount table is changed
-        boolean_t (*getBlockDiskActivity)(void *); // Disk activity callback: _getProcfsBlockDiskActivity (old kernels), _getSysfsBlockDiskActivity (new kernels)
-        boolean_t (*getCifsDiskActivity)(void *);  // Disk activity callback: _getCifsDiskActivity if /proc/fs/cifs/Stats is present, otherwise _getDummyDiskActivity
+        bool (*getBlockDiskActivity)(void *); // Disk activity callback: _getProcfsBlockDiskActivity (old kernels), _getSysfsBlockDiskActivity (new kernels)
+        bool (*getCifsDiskActivity)(void *);  // Disk activity callback: _getCifsDiskActivity if /proc/fs/cifs/Stats is present, otherwise _getDummyDiskActivity
 } _statistics = {};
 
 
 /* ----------------------------------------------------------------- Private */
 
 
-static boolean_t _getDiskUsage(void *_inf) {
+static bool _getDiskUsage(void *_inf) {
         Info_T inf = _inf;
         struct statvfs usage;
         if (statvfs(inf->filesystem->object.mountpoint, &usage) != 0) {
@@ -115,21 +120,21 @@ static boolean_t _getDiskUsage(void *_inf) {
 }
 
 
-static boolean_t _getDummyDiskActivity(void *_inf) {
+static bool _getDummyDiskActivity(__attribute__ ((unused)) void *_inf) {
         return true;
 }
 
 
-static boolean_t _getCifsDiskActivity(void *_inf) {
+static bool _getCifsDiskActivity(void *_inf) {
         Info_T inf = _inf;
         FILE *f = fopen(CIFSSTAT, "r");
         if (! f) {
                 LogError("Cannot open %s\n", CIFSSTAT);
                 return false;
         }
-        uint64_t now = Time_milli();
+        unsigned long long now = Time_milli();
         char line[PATH_MAX];
-        boolean_t found = false;
+        bool found = false;
         while (fgets(line, sizeof(line), f)) {
                 if (! found) {
                         int index;
@@ -140,9 +145,9 @@ static boolean_t _getCifsDiskActivity(void *_inf) {
                 } else if (found) {
                         char label1[256];
                         char label2[256];
-                        uint64_t operations;
-                        uint64_t bytes;
-                        if (sscanf(line, "%255[^:]: %"PRIu64" %255[^:]: %"PRIu64, label1, &operations, label2, &bytes) == 4) {
+                        unsigned long long operations;
+                        unsigned long long bytes;
+                        if (sscanf(line, "%255[^:]: %llu %255[^:]: %llu", label1, &operations, label2, &bytes) == 4) {
                                 if (Str_isEqual(label1, "Reads") && Str_isEqual(label2, "Bytes")) {
                                         Statistics_update(&(inf->filesystem->read.bytes), now, bytes);
                                         Statistics_update(&(inf->filesystem->read.operations), now, operations);
@@ -159,28 +164,28 @@ static boolean_t _getCifsDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _getNfsDiskActivity(void *_inf) {
+static bool _getNfsDiskActivity(void *_inf) {
         Info_T inf = _inf;
         FILE *f = fopen(NFSSTAT, "r");
         if (! f) {
                 LogError("Cannot open %s\n", NFSSTAT);
                 return false;
         }
-        uint64_t now = Time_milli();
+        unsigned long long now = Time_milli();
         char line[PATH_MAX];
-        char pattern[PATH_MAX];
-        boolean_t found = false;
+        char pattern[2 * PATH_MAX];
+        bool found = false;
         snprintf(pattern, sizeof(pattern), "device %s ", inf->filesystem->object.device);
         while (fgets(line, sizeof(line), f)) {
                 if (! found && Str_startsWith(line, pattern)) {
                         found = true;
                 } else if (found) {
                         char name[256];
-                        uint64_t operations;
-                        uint64_t bytesSent;
-                        uint64_t bytesReceived;
-                        uint64_t time;
-                        if (sscanf(line, " %255[^:]: %"PRIu64" %*u %*u %"PRIu64 " %"PRIu64" %*u %*u %"PRIu64, name, &operations, &bytesSent, &bytesReceived, &time) == 5) {
+                        unsigned long long operations;
+                        unsigned long long bytesSent;
+                        unsigned long long bytesReceived;
+                        unsigned long long time;
+                        if (sscanf(line, " %255[^:]: %llu %*u %*u %llu %llu %*u %*u %llu", name, &operations, &bytesSent, &bytesReceived, &time) == 5) {
                                 if (Str_isEqual(name, "READ")) {
                                         Statistics_update(&(inf->filesystem->time.read), now, time / 1000.); // us -> ms
                                         Statistics_update(&(inf->filesystem->read.bytes), now, bytesReceived);
@@ -199,19 +204,19 @@ static boolean_t _getNfsDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _getZfsDiskActivity(void *_inf) {
+static bool _getZfsDiskActivity(void *_inf) {
         Info_T inf = _inf;
-        char path[PATH_MAX];
+        char path[2 * PATH_MAX];
         snprintf(path, sizeof(path), "/proc/spl/kstat/zfs/%s/io", inf->filesystem->object.key);
         FILE *f = fopen(path, "r");
         if (f) {
                 char line[STRLEN];
-                uint64_t now = Time_milli();
-                uint64_t waitTime = 0ULL, runTime = 0ULL;
-                uint64_t readOperations = 0ULL, readBytes = 0ULL;
-                uint64_t writeOperations = 0ULL, writeBytes = 0ULL;
+                unsigned long long now = Time_milli();
+                unsigned long long waitTime = 0ULL, runTime = 0ULL;
+                unsigned long long readOperations = 0ULL, readBytes = 0ULL;
+                unsigned long long writeOperations = 0ULL, writeBytes = 0ULL;
                 while (fgets(line, sizeof(line), f)) {
-                        if (sscanf(line, "%"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %*u %*u %"PRIu64"", &readBytes, &writeBytes, &readOperations, &writeOperations, &waitTime, &runTime) == 6) {
+                        if (sscanf(line, "%llu %llu %llu %llu %llu %*u %*u %llu", &readBytes, &writeBytes, &readOperations, &writeOperations, &waitTime, &runTime) == 6) {
                                 Statistics_update(&(inf->filesystem->read.bytes), now, readBytes);
                                 Statistics_update(&(inf->filesystem->read.operations), now, readOperations);
                                 Statistics_update(&(inf->filesystem->write.bytes), now, writeBytes);
@@ -229,16 +234,88 @@ static boolean_t _getZfsDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _getSysfsBlockDiskActivity(void *_inf) {
+static bool _getVxfsDiskActivity(void *_inf) {
         Info_T inf = _inf;
+
+        // Get the major and minor node number to find the statistic data.
+        struct stat statbuf;
+        unsigned int st_major, st_minor;
+        unsigned int major = 0, minor = 0;
+
+        if (stat(inf->filesystem->object.device, &statbuf) < 0) {
+                return false;
+        }
+        st_major = major(statbuf.st_rdev);
+        st_minor = minor(statbuf.st_rdev);
+
+        // Try to find the statistic data in the sysfs first.
         char path[PATH_MAX];
+        snprintf(path, sizeof(path), "/sys/dev/block/%u:%u/stat", st_major, st_minor);
+        FILE *f = fopen(path, "r");
+        if (f) {
+                unsigned long long now = Time_milli();
+                unsigned long long readOperations = 0ULL, readSectors = 0ULL, readTime = 0ULL;
+                unsigned long long writeOperations = 0ULL, writeSectors = 0ULL, writeTime = 0ULL;
+                if (fscanf(f, "%llu %*u %llu %llu %llu %*u %llu %llu %*u %*u %*u", &readOperations, &readSectors, &readTime, &writeOperations, &writeSectors, &writeTime) != 6) {
+                        LogError("filesystem statistic error: cannot parse %s -- %s\n", path, STRERROR);
+                        fclose(f);
+                        return false;
+                }
+                Statistics_update(&(inf->filesystem->time.read), now, readTime);
+                Statistics_update(&(inf->filesystem->read.bytes), now, readSectors * 512);
+                Statistics_update(&(inf->filesystem->read.operations), now, readOperations);
+                Statistics_update(&(inf->filesystem->time.write), now, writeTime);
+                Statistics_update(&(inf->filesystem->write.bytes), now, writeSectors * 512);
+                Statistics_update(&(inf->filesystem->write.operations), now, writeOperations);
+                fclose(f);
+                return true;
+        }
+        DEBUG("filesystem statistic error: cannot read %s -- %s\n", path, STRERROR);
+
+        // Use the procfs file for backup purpose only.
+        // It should not used as main data collector, it support kernels >= 2.6.25 format only, too.
+        f = fopen(DISKSTAT, "r");
+        if (f) {
+                unsigned long long now = Time_milli();
+                unsigned long long readOperations = 0ULL, readSectors = 0ULL, readTime = 0ULL;
+                unsigned long long writeOperations = 0ULL, writeSectors = 0ULL, writeTime = 0ULL;
+                char line[PATH_MAX];
+                while (fgets(line, sizeof(line), f)) {
+                        char name[256] = {};
+
+                        // Traverse the /proc/diskstats to have the statistics.
+                        // The kernels >= 2.6.25 format with 11 fields is supported only.
+                        if (fscanf(f, "%u %u %255s %llu %*u %llu %llu %llu %*u %llu %llu %*u %*u %*u", &major, &minor, name, &readOperations, &readSectors, &readTime, &writeOperations, &writeSectors, &writeTime) == 9 && major == st_major && minor == st_minor) {
+                                Statistics_update(&(inf->filesystem->time.read), now, readTime);
+                                Statistics_update(&(inf->filesystem->read.bytes), now, readSectors * 512);
+                                Statistics_update(&(inf->filesystem->read.operations), now, readOperations);
+                                Statistics_update(&(inf->filesystem->time.write), now, writeTime);
+                                Statistics_update(&(inf->filesystem->write.bytes), now, writeSectors * 512);
+                                Statistics_update(&(inf->filesystem->write.operations), now, writeOperations);
+                                fclose(f);
+                                return true;
+                        }
+                }
+                fclose(f);
+                DEBUG("filesystem statistic error: cannot find device number %u %u for %s\n", st_major, st_minor, inf->filesystem->object.device);
+        } else {
+                DEBUG("filesystem statistic error: cannot read %s -- %s\n", DISKSTAT, STRERROR);
+        }
+        // Always true to get the disk usage, at least.
+        return true;
+}
+
+
+static bool _getSysfsBlockDiskActivity(void *_inf) {
+        Info_T inf = _inf;
+        char path[2 * PATH_MAX];
         snprintf(path, sizeof(path), "/sys/class/block/%s/stat", inf->filesystem->object.key);
         FILE *f = fopen(path, "r");
         if (f) {
-                uint64_t now = Time_milli();
-                uint64_t readOperations = 0ULL, readSectors = 0ULL, readTime = 0ULL;
-                uint64_t writeOperations = 0ULL, writeSectors = 0ULL, writeTime = 0ULL;
-                if (fscanf(f, "%"PRIu64" %*u %"PRIu64" %"PRIu64" %"PRIu64" %*u %"PRIu64" %"PRIu64" %*u %*u %*u", &readOperations, &readSectors, &readTime, &writeOperations, &writeSectors, &writeTime) != 6) {
+                unsigned long long now = Time_milli();
+                unsigned long long readOperations = 0ULL, readSectors = 0ULL, readTime = 0ULL;
+                unsigned long long writeOperations = 0ULL, writeSectors = 0ULL, writeTime = 0ULL;
+                if (fscanf(f, "%llu %*u %llu %llu %llu %*u %llu %llu %*u %*u %*u", &readOperations, &readSectors, &readTime, &writeOperations, &writeSectors, &writeTime) != 6) {
                         fclose(f);
                         LogError("filesystem statistic error: cannot parse %s -- %s\n", path, STRERROR);
                         return false;
@@ -257,20 +334,20 @@ static boolean_t _getSysfsBlockDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _getProcfsBlockDiskActivity(void *_inf) {
+static bool _getProcfsBlockDiskActivity(void *_inf) {
         Info_T inf = _inf;
         FILE *f = fopen(DISKSTAT, "r");
         if (f) {
-                uint64_t now = Time_milli();
-                uint64_t readOperations = 0ULL, readSectors = 0ULL;
-                uint64_t writeOperations = 0ULL, writeSectors = 0ULL;
+                unsigned long long now = Time_milli();
+                unsigned long long readOperations = 0ULL, readSectors = 0ULL;
+                unsigned long long writeOperations = 0ULL, writeSectors = 0ULL;
                 char line[PATH_MAX];
                 while (fgets(line, sizeof(line), f)) {
                         char name[256] = {};
                         // Fallback for kernels < 2.6.25: the /proc/diskstats used to have just 4 statistics, the file is present on >= 2.6.25 too and has 11 fields (same format as /sys/class/block/<NAME>/stat), but we use sysfs for it
                         // as we read the given partition directly instead of traversing the whole filesystems list. In this function we expect the old 4-statistics format - if it should be ever used as main data collector, it needs to
                         // be modified to support >= 2.6.25 format too.
-                        if (fscanf(f, " %*d %*d %255s %"PRIu64" %"PRIu64" %"PRIu64" %"PRIu64, name, &readOperations, &readSectors, &writeOperations, &writeSectors) == 5 && Str_isEqual(name, inf->filesystem->object.key)) {
+                        if (fscanf(f, " %*d %*d %255s %llu %llu %llu %llu", name, &readOperations, &readSectors, &writeOperations, &writeSectors) == 5 && Str_isEqual(name, inf->filesystem->object.key)) {
                                 Statistics_update(&(inf->filesystem->read.bytes), now, readSectors * 512);
                                 Statistics_update(&(inf->filesystem->read.operations), now, readOperations);
                                 Statistics_update(&(inf->filesystem->write.bytes), now, writeSectors * 512);
@@ -286,26 +363,26 @@ static boolean_t _getProcfsBlockDiskActivity(void *_inf) {
 }
 
 
-static boolean_t _compareMountpoint(const char *mountpoint, struct mntent *mnt) {
+static bool _compareMountpoint(const char *mountpoint, struct mntent *mnt) {
         return IS(mountpoint, mnt->mnt_dir);
 }
 
 
-static boolean_t _compareDevice(const char *device, struct mntent *mnt) {
+static bool _compareDevice(const char *device, struct mntent *mnt) {
         char target[PATH_MAX] = {};
         // The device listed in /etc/mtab can be a device mapper symlink (e.g. /dev/mapper/centos-root -> /dev/dm-1) ... lookup the device as is first (support for NFS/CIFS/SSHFS/etc.) and fallback to realpath if it didn't match
         return (Str_isEqual(device, mnt->mnt_fsname) || (realpath(mnt->mnt_fsname, target) && Str_isEqual(device, target)));
 }
 
 
-static boolean_t _setDevice(Info_T inf, const char *path, boolean_t (*compare)(const char *path, struct mntent *mnt)) {
+static bool _setDevice(Info_T inf, const char *path, bool (*compare)(const char *path, struct mntent *mnt)) {
         FILE *f = setmntent(MOUNTS, "r");
         if (! f) {
                 LogError("Cannot open %s\n", MOUNTS);
                 return false;
         }
         inf->filesystem->object.generation = _statistics.generation;
-        boolean_t mounted = false;
+        bool mounted = false;
         struct mntent *mnt;
         char flags[STRLEN];
         while ((mnt = getmntent(f))) {
@@ -316,7 +393,7 @@ static boolean_t _setDevice(Info_T inf, const char *path, boolean_t (*compare)(c
                         snprintf(inf->filesystem->object.type, sizeof(inf->filesystem->object.type), "%s", mnt->mnt_type);
                         snprintf(flags, sizeof(flags), "%s", mnt->mnt_opts);
                         inf->filesystem->object.getDiskUsage = _getDiskUsage; // The disk usage method is common for all filesystem types
-                        inf->filesystem->object.getDiskActivity = _getDummyDiskActivity; // Set to dummy IO statistics method by default (can be overriden bellow if statistics method is available for this filesystem)
+                        inf->filesystem->object.getDiskActivity = _getDummyDiskActivity; // Set to dummy IO statistics method by default (can be overridden bellow if statistics method is available for this filesystem)
                         if (Str_startsWith(mnt->mnt_type, "nfs")) {
                                 // NFS
                                 inf->filesystem->object.getDiskActivity = _getNfsDiskActivity;
@@ -332,6 +409,11 @@ static boolean_t _setDevice(Info_T inf, const char *path, boolean_t (*compare)(c
                                 // Need base zpool name for /proc/spl/kstat/zfs/<NAME>/io lookup:
                                 snprintf(inf->filesystem->object.key, sizeof(inf->filesystem->object.key), "%s", inf->filesystem->object.device);
                                 Str_replaceChar(inf->filesystem->object.key, '/', 0);
+                        } else if (IS(mnt->mnt_type, "vxfs")) {
+                                // VXFS, Veritas FS
+                                inf->filesystem->object.getDiskActivity = _getVxfsDiskActivity;
+                                // Use device major and minor number lookup in sysfs or procfs.
+                                snprintf(inf->filesystem->object.key, sizeof(inf->filesystem->object.key), "%s", inf->filesystem->object.device);
                         } else {
                                 if (realpath(mnt->mnt_fsname, inf->filesystem->object.key)) {
                                         // Need base name for /sys/class/block/<NAME>/stat or /proc/diskstats lookup:
@@ -363,7 +445,7 @@ static boolean_t _setDevice(Info_T inf, const char *path, boolean_t (*compare)(c
 }
 
 
-static boolean_t _getDevice(Info_T inf, const char *path, boolean_t (*compare)(const char *path, struct mntent *mnt)) {
+static bool _getDevice(Info_T inf, const char *path, bool (*compare)(const char *path, struct mntent *mnt)) {
         // Mount/unmount notification: open the /proc/self/mounts file if we're in daemon mode and keep it open until monit
         // stops, so we can poll for mount table changes
         // FIXME: when libev is added register the mount table handler in libev and stop polling here
@@ -395,7 +477,7 @@ static boolean_t _getDevice(Info_T inf, const char *path, boolean_t (*compare)(c
 /* --------------------------------------- Static constructor and destructor */
 
 
-static void __attribute__ ((constructor)) _constructor() {
+static void __attribute__ ((constructor)) _constructor(void) {
         struct stat sb;
         _statistics.fd = -1;
         _statistics.generation++; // First generation
@@ -404,7 +486,7 @@ static void __attribute__ ((constructor)) _constructor() {
 }
 
 
-static void __attribute__ ((destructor)) _destructor() {
+static void __attribute__ ((destructor)) _destructor(void) {
         if (_statistics.fd > -1) {
                   close(_statistics.fd);
         }
@@ -414,14 +496,14 @@ static void __attribute__ ((destructor)) _destructor() {
 /* ------------------------------------------------------------------ Public */
 
 
-boolean_t Filesystem_getByMountpoint(Info_T inf, const char *path) {
+bool Filesystem_getByMountpoint(Info_T inf, const char *path) {
         ASSERT(inf);
         ASSERT(path);
         return _getDevice(inf, path, _compareMountpoint);
 }
 
 
-boolean_t Filesystem_getByDevice(Info_T inf, const char *path) {
+bool Filesystem_getByDevice(Info_T inf, const char *path) {
         ASSERT(inf);
         ASSERT(path);
         return _getDevice(inf, path, _compareDevice);
