@@ -140,7 +140,7 @@ char *file_findControlFile() {
 }
 
 
-boolean_t file_createPidFile(char *pidfile) {
+bool file_createPidFile(const char *pidfile) {
         ASSERT(pidfile);
         unlink(pidfile);
         FILE *F = fopen(pidfile, "w");
@@ -154,7 +154,7 @@ boolean_t file_createPidFile(char *pidfile) {
 }
 
 
-boolean_t file_checkStat(char *filename, char *description, int permmask) {
+bool file_checkStat(const char *filename, const char *description, mode_t permmask) {
         ASSERT(filename);
         ASSERT(description);
         errno = 0;
@@ -179,7 +179,7 @@ boolean_t file_checkStat(char *filename, char *description, int permmask) {
 }
 
 
-boolean_t file_checkQueueDirectory(char *path) {
+bool file_checkQueueDirectory(const char *path) {
         if (mkdir(path, 0700) < 0 && errno != EEXIST) {
                 LogError("Cannot create the event queue directory '%s' -- %s\n", path, STRERROR);
                 return false;
@@ -188,7 +188,7 @@ boolean_t file_checkQueueDirectory(char *path) {
 }
 
 
-boolean_t file_checkQueueLimit(char *path, int limit) {
+bool file_checkQueueLimit(const char *path, int limit) {
         if (limit >= 0) {
                 DIR *dir = opendir(path);
                 if (! dir) {
@@ -212,7 +212,7 @@ boolean_t file_checkQueueLimit(char *path, int limit) {
 }
 
 
-boolean_t file_writeQueue(FILE *file, void *data, size_t size) {
+bool file_writeQueue(FILE *file, const void *data, size_t size) {
         ASSERT(file);
         /* write size */
         size_t rv = fwrite(&size, 1, sizeof(size_t), file);
@@ -265,7 +265,7 @@ void *file_readQueue(FILE *file, size_t *size) {
 }
 
 
-boolean_t file_readProc(char *buf, int buf_size, char *name, int pid, int *bytes_read) {
+bool file_readProc(char *buf, int buf_size, const char *name, int pid, int *bytes_read) {
         ASSERT(buf);
         ASSERT(name);
 
@@ -281,7 +281,7 @@ boolean_t file_readProc(char *buf, int buf_size, char *name, int pid, int *bytes
                 return false;
         }
 
-        boolean_t rv = false;
+        bool rv = false;
         int bytes = (int)read(fd, buf, buf_size - 1);
         if (bytes >= 0) {
                 if (bytes_read)
