@@ -75,13 +75,15 @@
 #endif
 
 #include "monit.h"
-#include "net.h"
 #include "ProcessTree.h"
 #include "state.h"
 #include "event.h"
 #include "engine.h"
 #include "client.h"
 #include "MMonit.h"
+#include "md5.h"
+#include "sha1.h"
+#include "checksum.h"
 
 // libmonit
 #include "Bootstrap.h"
@@ -108,10 +110,10 @@
 
 static void  do_init(void);                   /* Initialize this application */
 static void  do_reinit(void);       /* Re-initialize the runtime application */
-static void  do_action(int, char **);    /* Dispatch to the submitted action */
-static void  do_exit(boolean_t);                           /* Finalize monit */
+static void  do_action(List_T);          /* Dispatch to the submitted action */
+static void  do_exit(bool);                           /* Finalize monit */
 static void  do_default(void);                          /* Do default action */
-static void  handle_options(int, char **);         /* Handle program options */
+static void  handle_options(int, char **, List_T); /* Handle program options */
 static void  help(void);             /* Print program help message to stdout */
 static void  version(void);                     /* Print version information */
 static void *heartbeat(void *args);              /* M/Monit heartbeat thread */
@@ -130,26 +132,25 @@ struct Run_T Run;                      /**< Struct holding runtime constants */
 Service_T servicelist;                /**< The service list (created in p.y) */
 Service_T servicelist_conf;   /**< The service list in conf file (c. in p.y) */
 ServiceGroup_T servicegrouplist;/**< The service group list (created in p.y) */
-SystemInfo_T systeminfo;                              /**< System infomation */
+SystemInfo_T systeminfo;                             /**< System information */
 
 Thread_T heartbeatThread;
 Sem_T    heartbeatCond;
 Mutex_T  heartbeatMutex;
-static volatile boolean_t heartbeatRunning = false;
+static volatile bool heartbeatRunning = false;
 
-char *actionnames[] = {"ignore", "alert", "restart", "stop", "exec", "unmonitor", "start", "monitor", ""};
-char *modenames[] = {"active", "passive"};
-char *onrebootnames[] = {"start", "nostart", "laststate"};
-char *checksumnames[] = {"UNKNOWN", "MD5", "SHA1"};
-char *operatornames[] = {"less than", "less than or equal to", "greater than", "greater than or equal to", "equal to", "not equal to", "changed"};
-char *operatorshortnames[] = {"<", "<=", ">", ">=", "=", "!=", "<>"};
-char *servicetypes[] = {"Filesystem", "Directory", "File", "Process", "Remote Host", "System", "Fifo", "Program", "Network"};
-char *pathnames[] = {"Path", "Path", "Path", "Pid file", "Path", "", "Path"};
-char *icmpnames[] = {"Reply", "", "", "Destination Unreachable", "Source Quench", "Redirect", "", "", "Ping", "", "", "Time Exceeded", "Parameter Problem", "Timestamp Request", "Timestamp Reply", "Information Request", "Information Reply", "Address Mask Request", "Address Mask Reply"};
-char *sslnames[] = {"auto", "v2", "v3", "tlsv1", "tlsv1.1", "tlsv1.2", "tlsv1.3"};
-char *socketnames[] = {"unix", "IP", "IPv4", "IPv6"};
-char *timestampnames[] = {"modify/change time", "access time", "change time", "modify time"};
-char *httpmethod[] = {"", "HEAD", "GET"};
+const char *actionnames[] = {"ignore", "alert", "restart", "stop", "exec", "unmonitor", "start", "monitor", ""};
+const char *modenames[] = {"active", "passive"};
+const char *onrebootnames[] = {"start", "nostart", "laststate"};
+const char *checksumnames[] = {"UNKNOWN", "MD5", "SHA1"};
+const char *operatornames[] = {"less than", "less than or equal to", "greater than", "greater than or equal to", "equal to", "not equal to", "changed"};
+const char *operatorshortnames[] = {"<", "<=", ">", ">=", "=", "!=", "<>"};
+const char *servicetypes[] = {"Filesystem", "Directory", "File", "Process", "Remote Host", "System", "Fifo", "Program", "Network"};
+const char *pathnames[] = {"Path", "Path", "Path", "Pid file", "Path", "", "Path"};
+const char *icmpnames[] = {"Reply", "", "", "Destination Unreachable", "Source Quench", "Redirect", "", "", "Ping", "", "", "Time Exceeded", "Parameter Problem", "Timestamp Request", "Timestamp Reply", "Information Request", "Information Reply", "Address Mask Request", "Address Mask Reply"};
+const char *socketnames[] = {"unix", "IP", "IPv4", "IPv6"};
+const char *timestampnames[] = {"modify/change time", "access time", "change time", "modify time"};
+const char *httpmethod[] = {"", "HEAD", "GET"};
 
 
 /* ------------------------------------------------------------------ Public */
@@ -168,9 +169,11 @@ int main(int argc, char **argv) {
         Ssl_start();
 #endif
         init_env();
-        handle_options(argc, argv);
+        List_T arguments = List_new();
+        handle_options(argc, argv, arguments);
         do_init();
-        do_action(argc, argv);
+        do_action(arguments);
+        List_free(&arguments);
         do_exit(false);
         return 0;
 }
@@ -180,7 +183,7 @@ int main(int argc, char **argv) {
  * Wakeup a sleeping monit daemon.
  * Returns true on success otherwise false
  */
-boolean_t do_wakeupcall() {
+bool do_wakeupcall() {
         pid_t pid;
 
         if ((pid = exist_daemon()) > 0) {
@@ -194,7 +197,7 @@ boolean_t do_wakeupcall() {
 }
 
 
-boolean_t interrupt() {
+bool interrupt() {
         return Run.flags & Run_Stopped || Run.flags & Run_DoReload;
 }
 
@@ -202,7 +205,7 @@ boolean_t interrupt() {
 /* ----------------------------------------------------------------- Private */
 
 
-static void _validateOnce() {
+static void _validateOnce(void) {
         if (State_open()) {
                 State_restore();
                 validate();
@@ -334,7 +337,7 @@ static void do_reinit() {
          reinitialize any information about children we have setup to wait
          for will be lost. This may create zombie processes until Monit
          itself exit. However, Monit will wait on all children that has exited
-         before it ifself exit. TODO: Later refactored versions will use a
+         before it itself exit. TODO: Later refactored versions will use a
          globale process table which a sigchld handler can check */
         waitforchildren();
 
@@ -405,8 +408,8 @@ static void do_reinit() {
 /**
  * Dispatch to the submitted action - actions are program arguments
  */
-static void do_action(int argc, char **args) {
-        char *action = args[optind];
+static void do_action(List_T arguments) {
+        char *action = List_pop(arguments);
 
         Run.flags |= Run_Once;
 
@@ -417,7 +420,7 @@ static void do_action(int argc, char **args) {
                    IS(action, "monitor")   ||
                    IS(action, "unmonitor") ||
                    IS(action, "restart")) {
-                char *service = args[++optind];
+                char *service = List_pop(arguments);
                 if (Run.mygroup || service) {
                         int errors = 0;
                         List_T services = List_new();
@@ -454,19 +457,19 @@ static void do_action(int argc, char **args) {
                 LogInfo("Reinitializing %s daemon\n", prog);
                 kill_daemon(SIGHUP);
         } else if (IS(action, "status")) {
-                char *service = args[++optind];
+                char *service = List_pop(arguments);
                 if (! HttpClient_status(Run.mygroup, service))
                         exit(1);
         } else if (IS(action, "summary")) {
-                char *service = args[++optind];
+                char *service = List_pop(arguments);
                 if (! HttpClient_summary(Run.mygroup, service))
                         exit(1);
         } else if (IS(action, "report")) {
-                char *type = args[++optind];
+                char *type = List_pop(arguments);
                 if (! HttpClient_report(type))
                         exit(1);
         } else if (IS(action, "procmatch")) {
-                char *pattern = args[++optind];
+                char *pattern = List_pop(arguments);
                 if (! pattern) {
                         printf("Invalid syntax - usage: procmatch \"<pattern>\"\n");
                         exit(1);
@@ -476,7 +479,7 @@ static void do_action(int argc, char **args) {
                 kill_daemon(SIGTERM);
         } else if (IS(action, "validate")) {
                 if (do_wakeupcall()) {
-                        char *service = args[++optind];
+                        char *service = List_pop(arguments);
                         HttpClient_status(Run.mygroup, service);
                 } else {
                         _validateOnce();
@@ -492,7 +495,7 @@ static void do_action(int argc, char **args) {
 /**
  * Finalize monit
  */
-static void do_exit(boolean_t saveState) {
+static void do_exit(bool saveState) {
         set_signal_block();
         Run.flags |= Run_Stopped;
         if ((Run.flags & Run_Daemon) && ! (Run.flags & Run_Once)) {
@@ -524,7 +527,7 @@ static void do_exit(boolean_t saveState) {
 /**
  * Default action - become a daemon if defined in the Run object and
  * run validate() between sleeps. If not, just run validate() once.
- * Also, if specified, start the monit http server if in deamon mode.
+ * Also, if specified, start the monit http server if in daemon mode.
  */
 static void do_default() {
         if (Run.flags & Run_Daemon) {
@@ -611,42 +614,44 @@ static void do_default() {
  * Handle program options - Options set from the commandline
  * takes precedence over those found in the control file
  */
-static void handle_options(int argc, char **argv) {
+static void handle_options(int argc, char **argv, List_T arguments) {
         int opt;
         int deferred_opt = 0;
         opterr = 0;
         Run.mygroup = NULL;
-        const char *shortopts = "c:d:g:l:p:s:HIirtvVhB";
+        const char *shortopts = "+c:d:g:l:p:s:HIirtvVhB";
+        while (optind < argc) {
 #ifdef HAVE_GETOPT_LONG
-        struct option longopts[] = {
-                {"conf",        required_argument,      NULL,   'c'},
-                {"daemon",      required_argument,      NULL,   'd'},
-                {"group",       required_argument,      NULL,   'g'},
-                {"logfile",     required_argument,      NULL,   'l'},
-                {"pidfile",     required_argument,      NULL,   'p'},
-                {"statefile",   required_argument,      NULL,   's'},
-                {"hash",        optional_argument,      NULL,   'H'},
-                {"id",          no_argument,            NULL,   'i'},
-                {"help",        no_argument,            NULL,   'h'},
-                {"resetid",     no_argument,            NULL,   'r'},
-                {"test",        no_argument,            NULL,   't'},
-                {"verbose",     no_argument,            NULL,   'v'},
-                {"batch",       no_argument,            NULL,   'B'},
-                {"interactive", no_argument,            NULL,   'I'},
-                {"version",     no_argument,            NULL,   'V'},
-                {0}
-        };
-        while ((opt = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1)
+                struct option longopts[] = {
+                        {"conf",        required_argument,      NULL,   'c'},
+                        {"daemon",      required_argument,      NULL,   'd'},
+                        {"group",       required_argument,      NULL,   'g'},
+                        {"logfile",     required_argument,      NULL,   'l'},
+                        {"pidfile",     required_argument,      NULL,   'p'},
+                        {"statefile",   required_argument,      NULL,   's'},
+                        {"hash",        optional_argument,      NULL,   'H'},
+                        {"id",          no_argument,            NULL,   'i'},
+                        {"help",        no_argument,            NULL,   'h'},
+                        {"resetid",     no_argument,            NULL,   'r'},
+                        {"test",        no_argument,            NULL,   't'},
+                        {"verbose",     no_argument,            NULL,   'v'},
+                        {"batch",       no_argument,            NULL,   'B'},
+                        {"interactive", no_argument,            NULL,   'I'},
+                        {"version",     no_argument,            NULL,   'V'},
+                        {0}
+                };
+                if ((opt = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1)
 #else
-                while ((opt = getopt(argc, argv, shortopts)) != -1)
+                if ((opt = getopt(argc, argv, shortopts)) != -1)
 #endif
                 {
                         switch (opt) {
                                 case 'c':
                                 {
                                         char *f = optarg;
+                                        char realpath[PATH_MAX] = {};
                                         if (f[0] != SEPARATOR_CHAR)
-                                                f = File_getRealPath(optarg, (char[PATH_MAX]){});
+                                                f = File_getRealPath(optarg, realpath);
                                         if (! f)
                                                 THROW(AssertException, "The control file '%s' does not exist at %s",
                                                       Str_trunc(optarg, 80), Dir_cwd((char[STRLEN]){}, STRLEN));
@@ -717,9 +722,9 @@ static void handle_options(int argc, char **argv) {
                                 case 'H':
                                 {
                                         if (argc > optind)
-                                                Util_printHash(argv[optind]);
+                                                Checksum_printHash(argv[optind]);
                                         else
-                                                Util_printHash(NULL);
+                                                Checksum_printHash(NULL);
                                         exit(0);
                                         break;
                                 }
@@ -761,7 +766,10 @@ static void handle_options(int argc, char **argv) {
                                         exit(1);
                                 }
                         }
+                } else {
+                        List_append(arguments, argv[optind++]);
                 }
+        }
         /* Handle deferred options to make arguments to the program positional
          independent. These options are handled last, here as they represent exit
          points in the application and the control-file might be set with -c and
@@ -821,7 +829,7 @@ static void help() {
                " -v            Verbose mode, work noisy (diagnostic output)\n"
                " -vv           Very verbose mode, same as -v plus log stacktrace on error\n"
                " -H [filename] Print SHA1 and MD5 hashes of the file or of stdin if the\n"
-               "               filename is omited; monit will exit afterwards\n"
+               "               filename is omitted; monit will exit afterwards\n"
                " -V            Print version number and patchlevel\n"
                " -h            Print this text\n"
                "Optional commands are as follows:\n"
@@ -871,14 +879,14 @@ static void version() {
         printf("out");
 #endif
         printf(" large files\n");
-        printf("Copyright (C) 2001-2019 Tildeslash Ltd. All Rights Reserved.\n");
+        printf("Copyright (C) 2001-2020 Tildeslash Ltd. All Rights Reserved.\n");
 }
 
 
 /**
  * M/Monit heartbeat thread
  */
-static void *heartbeat(void *args) {
+static void *heartbeat(__attribute__ ((unused)) void *args) {
         set_signal_block();
         LogInfo("M/Monit heartbeat started\n");
         LOCK(heartbeatMutex)
@@ -901,7 +909,7 @@ static void *heartbeat(void *args) {
 /**
  * Signalhandler for a daemon reload call
  */
-static RETSIGTYPE do_reload(int sig) {
+static RETSIGTYPE do_reload(__attribute__ ((unused)) int sig) {
         Run.flags |= Run_DoReload;
 }
 
@@ -909,7 +917,7 @@ static RETSIGTYPE do_reload(int sig) {
 /**
  * Signalhandler for monit finalization
  */
-static RETSIGTYPE do_destroy(int sig) {
+static RETSIGTYPE do_destroy(__attribute__ ((unused)) int sig) {
         Run.flags |= Run_Stopped;
 }
 
@@ -917,7 +925,7 @@ static RETSIGTYPE do_destroy(int sig) {
 /**
  * Signalhandler for a daemon wakeup call
  */
-static RETSIGTYPE do_wakeup(int sig) {
+static RETSIGTYPE do_wakeup(__attribute__ ((unused)) int sig) {
         Run.flags |= Run_DoWakeup;
 }
 
