@@ -89,52 +89,32 @@
 #include <sys/socket.h>
 #endif
 
-#ifdef HAVE_MACH_BOOLEAN_H
-#include <mach/boolean.h>
-#endif
 #ifdef HAVE_UVM_UVM_PARAM_H
 #include <uvm/uvm_param.h>
 #endif
 #ifdef HAVE_VM_VM_H
 #include <vm/vm.h>
 #endif
-#ifdef HAVE_INTTYPES_H
-#include <inttypes.h>
-#else
-#define PRIu64 "llu"
-#endif
 
-
-//FIXME: we can export this type in libmonit
-#ifndef HAVE_BOOLEAN_T
-#undef true
-#undef false
-typedef enum {
-        false = 0,
-        true
-} __attribute__((__packed__)) boolean_t;
-#else
-#define false 0
-#define true  1
-#endif
+#include <stdbool.h>
 
 
 #include "Ssl.h"
 #include "Address.h"
-
+#include "statistics/Statistics.h"
+#include "net/socket.h"
+#include "net/Link.h"
 
 // libmonit
 #include "system/Command.h"
 #include "system/Process.h"
 #include "util/Str.h"
 #include "util/StringBuffer.h"
-#include "system/Link.h"
-#include "statistics/Statistics.h"
 #include "thread/Thread.h"
 
 
 #define MONITRC            "monitrc"
-#define TIMEFORMAT         "%Z %b %e %T"
+#define TIMEFORMAT         "%Z %Y %b %e %T"
 #define STRERROR            strerror(errno)
 #define STRLEN             256
 #ifndef USEC_PER_SEC
@@ -325,13 +305,21 @@ typedef enum {
         Resource_CpuUser,
         Resource_CpuSystem,
         Resource_CpuWait,
+        Resource_CpuNice,
+        Resource_CpuHardIRQ,
+        Resource_CpuSoftIRQ,
+        Resource_CpuSteal,
+        Resource_CpuGuest,
+        Resource_CpuGuestNice,
         Resource_CpuPercentTotal,
         Resource_SwapPercent,
         Resource_SwapKbyte,
         Resource_Threads,
         Resource_ReadBytes,
+        Resource_ReadBytesPhysical,
         Resource_ReadOperations,
         Resource_WriteBytes,
+        Resource_WriteBytesPhysical,
         Resource_WriteOperations,
         Resource_ServiceTime,
         Resource_LoadAveragePerCore1m,
@@ -380,6 +368,22 @@ typedef enum {
 } __attribute__((__packed__)) MmonitCompress_Type;
 
 
+typedef enum {
+        Statistics_CpuUser                      = 0x1,
+        Statistics_CpuNice                      = 0x2,
+        Statistics_CpuSystem                    = 0x4,
+        Statistics_CpuIOWait                    = 0x8,
+        Statistics_CpuHardIRQ                   = 0x10,
+        Statistics_CpuSoftIRQ                   = 0x20,
+        Statistics_CpuSteal                     = 0x40,
+        Statistics_CpuGuest                     = 0x80,
+        Statistics_CpuGuestNice                 = 0x100,
+        Statistics_FiledescriptorsPerSystem     = 0x200,
+        Statistics_FiledescriptorsPerProcess    = 0x400,
+        Statistics_FiledescriptorsPerProcessMax = 0x800
+} __attribute__((__packed__)) Statistics_Flags;
+
+
 /* Length of the longest message digest in bytes */
 #define MD_SIZE 65
 
@@ -401,8 +405,6 @@ typedef enum {
 #define LIMIT_RESTARTTIMEOUT    30000
 
 
-#include "socket.h"
-
 
 /** ------------------------------------------------- Special purpose macros */
 
@@ -445,10 +447,10 @@ typedef char MD_T[MD_SIZE];
 
 /** Defines monit limits object */
 typedef struct Limits_T {
+        int      programOutput;           /**< Program output truncate limit [B] */
+        size_t   fileContentBuffer;  /**< Maximum tested file content length [B] */
         uint32_t sendExpectBuffer;  /**< Maximum send/expect response length [B] */
-        uint32_t fileContentBuffer;  /**< Maximum tested file content length [B] */
         uint32_t httpContentBuffer;  /**< Maximum tested HTTP content length [B] */
-        uint32_t programOutput;           /**< Program output truncate limit [B] */
         uint32_t networkTimeout;               /**< Default network timeout [ms] */
         uint32_t programTimeout;               /**< Default program timeout [ms] */
         uint32_t stopTimeout;                     /**< Default stop timeout [ms] */
@@ -462,26 +464,26 @@ typedef struct Limits_T {
  * array must be NULL terminated and the first entry is the program
  * itself. In addition, a user and group may be set for the Command
  * which means that the Command should run as a certain user and with
- * certain group. To avoid name collision with Command_T in libmonit 
+ * certain group. To avoid name collision with Command_T in libmonit
  * this structure uses lower case.
  */
 typedef struct command_t {
         char *arg[ARGMAX];                             /**< Program with arguments */
         short length;                       /**< The length of the arguments array */
-        boolean_t has_uid;      /**< true if a new uid is defined for this Command */
-        boolean_t has_gid;      /**< true if a new gid is defined for this Command */
+        bool has_uid;      /**< true if a new uid is defined for this Command */
+        bool has_gid;      /**< true if a new gid is defined for this Command */
         uid_t uid;         /**< The user id to switch to when running this Command */
         gid_t gid;        /**< The group id to switch to when running this Command */
-        unsigned timeout;     /**< Max seconds which we wait for method to execute */
+        unsigned int timeout;     /**< Max seconds which we wait for method to execute */
 } *command_t;
 
 
 /** Defines an event action object */
 typedef struct Action_T {
         Action_Type id;                                   /**< Action to be done */
-        uint8_t count;             /**< Event count needed to trigger the action */
-        uint8_t cycles;      /**< Cycles during which count limit can be reached */
-        uint8_t repeat;                         /*< Repeat action each Xth cycle */
+        int count;                 /**< Event count needed to trigger the action */
+        int cycles;          /**< Cycles during which count limit can be reached */
+        int repeat;                             /*< Repeat action each Xth cycle */
         command_t exec;                     /**< Optional command to be executed */
 } *Action_T;
 
@@ -503,7 +505,7 @@ typedef struct URL_T {
         char *path;                                        /**< URL path     part */
         char *query;                                       /**< URL query    part */
         int   port;                                        /**< URL port     part */
-        boolean_t ipv6;
+        bool ipv6;
 } *URL_T;
 
 
@@ -562,40 +564,53 @@ typedef struct Auth_T {
         char *passwd;                                /**< The users password data */
         char *groupname;                                      /**< PAM group name */
         Digest_Type digesttype;                /**< How did we store the password */
-        boolean_t is_readonly; /**< true if this is a read-only authenticated user*/
+        bool is_readonly; /**< true if this is a read-only authenticated user*/
         struct Auth_T *next;                 /**< Next credential or NULL if last */
 } *Auth_T;
 
 
 /** Defines data for systemwide statistic */
 typedef struct SystemInfo_T {
+        Statistics_Flags statisticsAvailable; /**< List of statistics that are available on this system */
         struct {
                 int count;                                      /**< Number of CPUs */
                 struct {
-                        float user;         /**< Total CPU in use in user space [%] */
-                        float system;     /**< Total CPU in use in kernel space [%] */
-                        float wait;            /**< Total CPU in use in waiting [%] */
+                        float user;       /**< Time in user space [%] */
+                        float nice;       /**< Time in user space with low priority [%] */
+                        float system;     /**< Time in kernel space [%] */
+                        float iowait;     /**< Idle time while waiting for I/O [%] */
+                        float hardirq;    /**< Time servicing hardware interrupts [%] */
+                        float softirq;    /**< Time servicing software interrupts [%] */
+                        float steal;      /**< Stolen time, which is the time spent in other operating systems when running in a virtualized environment [%] */
+                        float guest;      /**< Time spent running a virtual CPU for guest operating systems under the control of the kernel [%] */
+                        float guest_nice; /**< Time spent running a niced guest (virtual CPU for guest operating systems under the control of the kernel) [%] */
+                        float idle;       /**< Idle time [%] */
                 } usage;
         } cpu;
         struct {
-                uint64_t size;                      /**< Maximal system real memory */
+                unsigned long long size;                      /**< Maximal system real memory */
                 struct {
                         float percent;  /**< Total real memory in use in the system */
-                        uint64_t bytes; /**< Total real memory in use in the system */
+                        unsigned long long bytes; /**< Total real memory in use in the system */
                 } usage;
         } memory;
         struct {
-                uint64_t size;                                       /**< Swap size */
+                unsigned long long size;                                       /**< Swap size */
                 struct {
                         float percent;         /**< Total swap in use in the system */
-                        uint64_t bytes;        /**< Total swap in use in the system */
+                        unsigned long long bytes;        /**< Total swap in use in the system */
                 } usage;
         } swap;
+        struct {
+                long long allocated;        /**< Number of allocated filedescriptors */
+                long long unused;              /**< Number of unused filedescriptors */
+                long long maximum;                        /**< Filedescriptors limit */
+        } filedescriptors;
         size_t argmax;                                                   /**< Program arguments maximum [B] */
         double loadavg[3];                                                         /**< Load average triple */
         struct utsname uname;                                 /**< Platform information provided by uname() */
         struct timeval collected;                                             /**< When were data collected */
-        uint64_t booted; /**< System boot time (seconds since UNIX epoch, using platform-agnostic uint64_t) */
+        unsigned long long booted; /**< System boot time (seconds since UNIX epoch, using platform-agnostic unsigned long long) */
         double time;                                                                      /**< 1/10 seconds */
         double time_prev;                                                                 /**< 1/10 seconds */
 } SystemInfo_T;
@@ -650,7 +665,7 @@ typedef struct Port_T {
         Socket_Type type;           /**< Socket type used for connection (UDP/TCP) */
         Socket_Family family;    /**< Socket family used for connection (NET/UNIX) */
         Connection_State is_available;               /**< Server/port availability */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
         /** Protocol specific parameters */
         union {
                 struct {
@@ -683,7 +698,7 @@ typedef struct Port_T {
                 } generic;
                 struct {
                         Hash_Type hashtype;           /**< Type of hash for a checksum (optional) */
-                        boolean_t hasStatus;                    /**< Is explicit HTTP status set? */
+                        bool hasStatus;                    /**< Is explicit HTTP status set? */
                         Operator_Type operator;                         /**< HTTP status operator */
                         Http_Method method;
                         int status;                                              /**< HTTP status */
@@ -700,6 +715,8 @@ typedef struct Port_T {
                 struct {
                         char *username;
                         char *password;
+                        char *rsaChecksum;
+                        Hash_Type rsaChecksumType;
                 } mysql;
                 struct {
                         char *secret;
@@ -737,7 +754,7 @@ typedef struct Icmp_T {
         Socket_Family family;                 /**< ICMP family used for connection */
         double response;                         /**< ICMP ECHO response time [ms] */
         Outgoing_T outgoing;                                 /**< Outgoing address */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Icmp_T *next;                               /**< next icmp in chain */
@@ -746,7 +763,8 @@ typedef struct Icmp_T {
 
 typedef struct Dependant_T {
         char *dependant;                            /**< name of dependant service */
-        char *dependant_escaped;        /**< URL escaped name of dependant service */
+        char *dependant_urlescaped;     /**< URL escaped name of dependant service */
+        StringBuffer_T dependant_htmlescaped; /**< HTML escaped name of dependant service */
 
         /** For internal use */
         struct Dependant_T *next;             /**< next dependant service in chain */
@@ -758,7 +776,7 @@ typedef struct Resource_T {
         Resource_Type resource_id;                     /**< Which value is checked */
         Operator_Type operator;                           /**< Comparison operator */
         double limit;                                   /**< Limit of the resource */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Resource_T *next;                       /**< next resource in chain */
@@ -767,13 +785,13 @@ typedef struct Resource_T {
 
 /** Defines timestamp object */
 typedef struct Timestamp_T {
-        boolean_t initialized;              /**< true if timestamp was initialized */
-        boolean_t test_changes;       /**< true if we only should test for changes */
+        bool initialized;              /**< true if timestamp was initialized */
+        bool test_changes;       /**< true if we only should test for changes */
         Timestamp_Type type;
         Operator_Type operator;                           /**< Comparison operator */
-        uint64_t time;                                    /**< Timestamp watermark */
+        unsigned long long time;                                    /**< Timestamp watermark */
         time_t lastTimestamp;        /**< Last timestamp (context depends on type) */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Timestamp_T *next;                     /**< next timestamp in chain */
@@ -791,7 +809,7 @@ typedef struct ActionRate_T {
 } *ActionRate_T;
 
 
-/** Defines when to run a check for a service. This type suports both the old
+/** Defines when to run a check for a service. This type supports both the old
  cycle based every statement and the new cron-format version */
 typedef struct Every_T {
         Every_Type type; /**< 0 = not set, 1 = cycle, 2 = cron, 3 = negated cron */
@@ -807,10 +825,10 @@ typedef struct Every_T {
 
 
 typedef struct Status_T {
-        boolean_t initialized;                 /**< true if status was initialized */
+        bool initialized;                 /**< true if status was initialized */
         Operator_Type operator;                           /**< Comparison operator */
         int return_value;                /**< Return value of the program to check */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Status_T *next;                       /**< next exit value in chain */
@@ -831,11 +849,11 @@ typedef struct Program_T {
 
 /** Defines size object */
 typedef struct Size_T {
-        boolean_t initialized;                   /**< true if size was initialized */
-        boolean_t test_changes;       /**< true if we only should test for changes */
+        bool initialized;                   /**< true if size was initialized */
+        bool test_changes;       /**< true if we only should test for changes */
         Operator_Type operator;                           /**< Comparison operator */
         unsigned long long size;                               /**< Size watermark */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Size_T *next;                               /**< next size in chain */
@@ -846,7 +864,7 @@ typedef struct Size_T {
 typedef struct Uptime_T {
         Operator_Type operator;                           /**< Comparison operator */
         unsigned long long uptime;                           /**< Uptime watermark */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Uptime_T *next;                           /**< next uptime in chain */
@@ -854,7 +872,7 @@ typedef struct Uptime_T {
 
 
 typedef struct LinkStatus_T {
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct LinkStatus_T *next;                      /**< next link in chain */
@@ -864,7 +882,7 @@ typedef struct LinkStatus_T {
 typedef struct LinkSpeed_T {
         int duplex;                                        /**< Last duplex status */
         long long speed;                                     /**< Last speed [bps] */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct LinkSpeed_T *next;                       /**< next link in chain */
@@ -874,7 +892,7 @@ typedef struct LinkSpeed_T {
 typedef struct LinkSaturation_T {
         Operator_Type operator;                           /**< Comparison operator */
         float limit;                                     /**< Saturation limit [%] */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct LinkSaturation_T *next;                  /**< next link in chain */
@@ -886,7 +904,7 @@ typedef struct Bandwidth_T {
         Time_Type range;                            /**< Time range to watch: unit */
         int rangecount;                            /**< Time range to watch: count */
         unsigned long long limit;                              /**< Data watermark */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Bandwidth_T *next;                     /**< next bandwidth in chain */
@@ -895,31 +913,31 @@ typedef struct Bandwidth_T {
 
 /** Defines checksum object */
 typedef struct Checksum_T {
-        boolean_t initialized;               /**< true if checksum was initialized */
-        boolean_t test_changes;       /**< true if we only should test for changes */
+        bool initialized;               /**< true if checksum was initialized */
+        bool test_changes;       /**< true if we only should test for changes */
         Hash_Type type;                   /**< The type of hash (e.g. md5 or sha1) */
         int   length;                                      /**< Length of the hash */
         MD_T  hash;                     /**< A checksum hash computed for the path */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 } *Checksum_T;
 
 
 /** Defines permission object */
 typedef struct Perm_T {
-        boolean_t test_changes;       /**< true if we only should test for changes */
+        bool test_changes;       /**< true if we only should test for changes */
         int perm;                                           /**< Access permission */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 } *Perm_T;
 
 /** Defines match object */
 typedef struct Match_T {
-        boolean_t ignore;                                        /**< Ignore match */
-        boolean_t not;                                           /**< Invert match */
+        bool ignore;                                        /**< Ignore match */
+        bool not;                                           /**< Invert match */
         char    *match_string;                                   /**< Match string */ //FIXME: union?
         char    *match_path;                         /**< File with matching rules */ //FIXME: union?
         regex_t *regex_comp;                                    /**< Match compile */
-        StringBuffer_T log;    /**< The temporary buffer used to record the matches */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        StringBuffer_T log;   /**< The temporary buffer used to record the matches */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Match_T *next;                             /**< next match in chain */
@@ -929,29 +947,39 @@ typedef struct Match_T {
 /** Defines uid object */
 typedef struct Uid_T {
         uid_t     uid;                                            /**< Owner's uid */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 } *Uid_T;
 
 
 /** Defines gid object */
 typedef struct Gid_T {
         gid_t     gid;                                            /**< Owner's gid */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 } *Gid_T;
 
 
 typedef struct SecurityAttribute_T {
         char *attribute;                                   /**< Security attribute */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct SecurityAttribute_T *next;
 } *SecurityAttribute_T;
 
+typedef struct Filedescriptors_T {
+        bool total;             /**<Whether to include filedescriptors of children */
+        long long limit_absolute;                      /**<  Filedescriptors limit */
+        float limit_percent;                            /**< Filedescriptors limit */
+        Operator_Type operator;                           /**< Comparison operator */
+        EventAction_T action; /**< Description of the action upon event occurrence */
+
+        /** For internal use */
+        struct Filedescriptors_T *next;
+} *Filedescriptors_T;
 
 /** Defines pid object */
 typedef struct Pid_T {
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Pid_T *next;                                 /**< next pid in chain */
@@ -959,7 +987,7 @@ typedef struct Pid_T {
 
 
 typedef struct FsFlag_T {
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct FsFlag_T *next;
@@ -967,7 +995,7 @@ typedef struct FsFlag_T {
 
 
 typedef struct NonExist_T {
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct NonExist_T *next;
@@ -975,7 +1003,7 @@ typedef struct NonExist_T {
 
 
 typedef struct Exist_T {
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct Exist_T *next;
@@ -989,7 +1017,7 @@ typedef struct FileSystem_T {
         //FIXME: union
         long long limit_absolute;                          /**< Watermark - blocks */
         float limit_percent;                              /**< Watermark - percent */
-        EventAction_T action;  /**< Description of the action upon event occurence */
+        EventAction_T action; /**< Description of the action upon event occurrence */
 
         /** For internal use */
         struct FileSystem_T *next;                   /**< next filesystem in chain */
@@ -997,13 +1025,14 @@ typedef struct FileSystem_T {
 
 
 typedef struct IOStatistics_T {
-        struct Statistics_T operations;        /**< Number of operations completed */
-        struct Statistics_T bytes;      /**< Number of bytes handled by operations */
+        struct Statistics_T operations;                                         /**< Number of operations completed */
+        struct Statistics_T bytes;          /**< Number of bytes handled by operations (total including cached I/O) */
+        struct Statistics_T bytesPhysical;           /**< Number of bytes handled by operations (physical I/O only) */
 } *IOStatistics_T;
 
 
 typedef struct Device_T {
-        boolean_t mounted;
+        bool mounted;
         int generation;
         int instance;
         char partition;
@@ -1012,16 +1041,16 @@ typedef struct Device_T {
         char key[PATH_MAX];
         char module[256];
         char type[64];
-        uint64_t flags;
-        boolean_t (*getDiskUsage)(void *);
-        boolean_t (*getDiskActivity)(void *);
+        unsigned long long flags;
+        bool (*getDiskUsage)(void *);
+        bool (*getDiskActivity)(void *);
 } *Device_T;
 
 
 typedef struct TimestampInfo_T {
-        uint64_t access;
-        uint64_t change;
-        uint64_t modify;
+        unsigned long long access;
+        unsigned long long change;
+        unsigned long long modify;
 } *TimestampInfo_T;
 
 
@@ -1040,7 +1069,7 @@ typedef struct FileSystemInfo_T {
         int gid;                                              /**< Owner's gid */
         int mode;                                              /**< Permission */
         char flags[STRLEN];                              /**< Filesystem flags */
-        boolean_t flagsChanged;          /**< True if filesystem flags changed */
+        bool flagsChanged;          /**< True if filesystem flags changed */
         struct IOStatistics_T read;                       /**< Read statistics */
         struct IOStatistics_T write;                     /**< Write statistics */
         struct {
@@ -1083,7 +1112,7 @@ typedef struct FifoInfo_T {
 
 
 typedef struct ProcessInfo_T {
-        boolean_t zombie;
+        bool zombie;
         pid_t _pid;                           /**< Process PID from last cycle */
         pid_t _ppid;                   /**< Process parent PID from last cycle */
         pid_t pid;                          /**< Process PID from actual cycle */
@@ -1093,8 +1122,8 @@ typedef struct ProcessInfo_T {
         int gid;                                              /**< Process GID */
         int threads;
         int children;
-        uint64_t mem;
-        uint64_t total_mem;
+        unsigned long long mem;
+        unsigned long long total_mem;
         float mem_percent;                                     /**< percentage */
         float total_mem_percent;                               /**< percentage */
         float cpu_percent;                                     /**< percentage */
@@ -1103,6 +1132,14 @@ typedef struct ProcessInfo_T {
         struct IOStatistics_T read;                       /**< Read statistics */
         struct IOStatistics_T write;                     /**< Write statistics */
         char secattr[STRLEN];                         /**< Security attributes */
+        struct {
+                long long open;                        /**< number of opened files */
+                long long openTotal;             /**< number of total opened files */
+                struct {
+                        long long soft;                 /**< Filedescriptors soft limit */
+                        long long hard;                 /**< Filedescriptors hard limit */
+                } limit;
+        } filedescriptors;
 } *ProcessInfo_T;
 
 
@@ -1129,9 +1166,11 @@ typedef struct Service_T {
 
         /** Common parameters */
         char *name;                                  /**< Service descriptive name */
-        char *name_escaped;                          /**< Service name URL escaped */
+        char *name_urlescaped;                       /**< Service name URL escaped */
+        StringBuffer_T name_htmlescaped;            /**< Service name HTML escaped */
         State_Type (*check)(struct Service_T *);/**< Service verification function */
-        boolean_t visited; /**< Service visited flag, set if dependencies are used */
+        bool onrebootRestored;
+        bool visited; /**< Service visited flag, set if dependencies are used */
         Service_Type type;                             /**< Monitored service type */
         Monitor_State monitor;                             /**< Monitor state flag */
         Monitor_Mode mode;                    /**< Monitoring mode for the service */
@@ -1156,7 +1195,7 @@ typedef struct Service_T {
         Perm_T      perm;                                    /**< Permission check */
         Port_T      portlist;                            /**< Portnumbers to check */
         Port_T      socketlist;                         /**< Unix sockets to check */
-        Resource_T  resourcelist;                          /**< Resouce check list */
+        Resource_T  resourcelist;                         /**< Resource check list */
         Size_T      sizelist;                                 /**< Size check list */
         Uptime_T    uptimelist;                             /**< Uptime check list */
         Match_T     matchlist;                             /**< Content Match list */
@@ -1172,6 +1211,7 @@ typedef struct Service_T {
         Uid_T       euid;                                 /**< Effective Uid check */
         Gid_T       gid;                                            /**< Gid check */
         SecurityAttribute_T secattrlist;             /**< Security attributes list */
+        Filedescriptors_T filedescriptorslist;                   /**< Filedescriptors list */
         LinkStatus_T linkstatuslist;                 /**< Network link status list */
         LinkSpeed_T linkspeedlist;                    /**< Network link speed list */
         LinkSaturation_T linksaturationlist;     /**< Network link saturation list */
@@ -1194,7 +1234,7 @@ typedef struct Service_T {
         int                error;                          /**< Error flags bitmap */
         int                error_hint;   /**< Failed/Changed hint for error bitmap */
         union Info_T       inf;                          /**< Service check result */
-        struct timeval     collected;                /**< When were data collected */ //FIXME: replace with uint64_t? (all places where timeval is used) ... Time_milli()?
+        struct timeval     collected;                /**< When were data collected */ //FIXME: replace with unsigned long long? (all places where timeval is used) ... Time_milli()?
         char              *token;                                /**< Action token */
 
         /** Events */
@@ -1206,7 +1246,7 @@ typedef struct Service_T {
                 Monitor_Mode      mode;             /**< Monitoring mode for the service */
                 Service_Type      type;                      /**< Monitored service type */
                 State_Type        state;                                 /**< Test state */
-                boolean_t         state_changed;              /**< true if state changed */
+                bool         state_changed;              /**< true if state changed */
                 Handler_Type      flag;                     /**< The handlers state flag */
                 long long         state_map;           /**< Event bitmap for last cycles */
                 unsigned int      count;                             /**< The event rate */
@@ -1256,7 +1296,7 @@ struct Run_T {
         MD_T id;                                              /**< Unique monit id */
         Limits_T limits;                                       /**< Default limits */
         struct SslOptions_T ssl;                          /**< Default SSL options */
-        int  polltime;        /**< In deamon mode, the sleeptime (sec) between run */
+        int  polltime;        /**< In daemon mode, the sleeptime (sec) between run */
         int  startdelay;  /**< the sleeptime [s] on first start after machine boot */
         int  facility;              /** The facility to use when running openlog() */
         int  eventlist_slots;          /**< The event queue size - number of slots */
@@ -1319,19 +1359,18 @@ extern Service_T      servicelist_conf;
 extern ServiceGroup_T servicegrouplist;
 extern SystemInfo_T   systeminfo;
 
-extern char *actionnames[];
-extern char *modenames[];
-extern char *onrebootnames[];
-extern char *checksumnames[];
-extern char *operatornames[];
-extern char *operatorshortnames[];
-extern char *servicetypes[];
-extern char *pathnames[];
-extern char *icmpnames[];
-extern char *sslnames[];
-extern char *socketnames[];
-extern char *timestampnames[];
-extern char *httpmethod[];
+extern const char *actionnames[];
+extern const char *modenames[];
+extern const char *onrebootnames[];
+extern const char *checksumnames[];
+extern const char *operatornames[];
+extern const char *operatorshortnames[];
+extern const char *servicetypes[];
+extern const char *pathnames[];
+extern const char *icmpnames[];
+extern const char *socketnames[];
+extern const char *timestampnames[];
+extern const char *httpmethod[];
 
 
 /* ------------------------------------------------------- Public prototypes */
@@ -1343,13 +1382,13 @@ extern char *httpmethod[];
 #include "system/Mem.h"
 
 
-/* FIXME: move remaining prototypes into seperate header-files */
+/* FIXME: move remaining prototypes into separate header-files */
 
-boolean_t parse(char *);
-boolean_t control_service(const char *, Action_Type);
-boolean_t control_service_string(List_T, const char *);
+bool parse(char *);
+bool control_service(const char *, Action_Type);
+bool control_service_string(List_T, const char *);
 void  spawn(Service_T, command_t, Event_T);
-boolean_t log_init(void);
+bool log_init(void);
 void  LogEmergency(const char *, ...) __attribute__((format (printf, 1, 2)));
 void  LogAlert(const char *, ...) __attribute__((format (printf, 1, 2)));
 void  LogCritical(const char *, ...) __attribute__((format (printf, 1, 2)));
@@ -1358,15 +1397,15 @@ void  LogWarning(const char *, ...) __attribute__((format (printf, 1, 2)));
 void  LogNotice(const char *, ...) __attribute__((format (printf, 1, 2)));
 void  LogInfo(const char *, ...) __attribute__((format (printf, 1, 2)));
 void  LogDebug(const char *, ...) __attribute__((format (printf, 1, 2)));
-void  vLogEmergency(const char *, va_list ap);
-void  vLogAlert(const char *, va_list ap);
-void  vLogCritical(const char *, va_list ap);
-void  vLogError(const char *, va_list ap);
-void  vLogWarning(const char *,va_list ap);
-void  vLogNotice(const char *, va_list ap);
-void  vLogInfo(const char *, va_list ap);
-void  vLogDebug(const char *, va_list ap);
-void  vLogAbortHandler(const char *s, va_list ap);
+void  vLogEmergency(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogAlert(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogCritical(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogError(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogWarning(const char *,va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogNotice(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogInfo(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogDebug(const char *, va_list ap) __attribute__((format (printf, 1, 0)));
+void  vLogAbortHandler(const char *s, va_list ap) __attribute__((format (printf, 1, 0))) __attribute__((noreturn));
 void  log_close(void);
 #ifndef HAVE_VSYSLOG
 #ifdef HAVE_SYSLOG
@@ -1379,12 +1418,12 @@ void  gc(void);
 void  gc_mail_list(Mail_T *);
 void  gccmd(command_t *);
 void  gc_event(Event_T *e);
-boolean_t kill_daemon(int);
+bool kill_daemon(int);
 int   exist_daemon(void);
-boolean_t sendmail(Mail_T);
+bool sendmail(Mail_T);
 void  init_env(void);
 void  monit_http(Httpd_Action);
-boolean_t can_http(void);
+bool can_http(void);
 void set_signal_block(void);
 State_Type check_process(Service_T);
 State_Type check_filesystem(Service_T);
@@ -1397,7 +1436,7 @@ State_Type check_program(Service_T);
 State_Type check_net(Service_T);
 int  check_URL(Service_T s);
 void status_xml(StringBuffer_T, Event_T, int, const char *);
-boolean_t  do_wakeupcall(void);
-boolean_t interrupt(void);
+bool  do_wakeupcall(void);
+bool interrupt(void);
 
 #endif
