@@ -73,13 +73,14 @@
 static int  pagesize;
 static long total_old    = 0;
 static long cpu_user_old = 0;
+static long cpu_nice_old = 0;
 static long cpu_syst_old = 0;
 
 
 /* ------------------------------------------------------------------ Public */
 
 
-boolean_t init_process_info_sysdep(void) {
+bool init_process_info_sysdep(void) {
         size_t size = sizeof(systeminfo.cpu.count);
         if (sysctlbyname("hw.logicalcpu", &systeminfo.cpu.count, &size, NULL, 0) == -1) {
                 DEBUG("system statistics error -- sysctl hw.logicalcpu failed: %s\n", STRERROR);
@@ -145,7 +146,7 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                 cmdline = StringBuffer_create(64);
                 args = CALLOC(1, systeminfo.argmax + 1);
         }
-        for (int i = 0; i < treesize; i++) {
+        for (size_t i = 0; i < treesize; i++) {
                 pt[i].uptime    = systeminfo.time / 10. - pinfo[i].kp_proc.p_starttime.tv_sec;
                 pt[i].zombie    = pinfo[i].kp_proc.p_stat == SZOMB ? true : false;
                 pt[i].pid       = pinfo[i].kp_proc.p_pid;
@@ -184,8 +185,13 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                                         pt[i].cmdline = Str_dup(StringBuffer_toString(StringBuffer_trim(cmdline)));
                         }
                         if (STR_UNDEF(pt[i].cmdline)) {
+                                char cmdpath[PROC_PIDPATHINFO_MAXSIZE] = {};
                                 FREE(pt[i].cmdline);
-                                pt[i].cmdline = Str_dup(pinfo[i].kp_proc.p_comm);
+                                if (proc_pidpath(pt[i].pid, cmdpath, sizeof(cmdpath)) > 0) {
+                                        pt[i].cmdline = Str_dup(cmdpath);
+                                } else {
+                                        pt[i].cmdline = Str_dup(pinfo[i].kp_proc.p_comm);
+                                }
                         }
                 }
                 if (! pt[i].zombie) {
@@ -195,10 +201,10 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                         if (rv <= 0) {
                                 if (errno != EPERM)
                                         DEBUG("proc_pidinfo for pid %d failed -- %s\n", pt[i].pid, STRERROR);
-                        } else if (rv < sizeof(tinfo)) {
+                        } else if ((unsigned long)rv < sizeof(tinfo)) {
                                 LogError("proc_pidinfo for pid %d -- invalid result size\n", pt[i].pid);
                         } else {
-                                pt[i].memory.usage = (uint64_t)tinfo.pti_resident_size;
+                                pt[i].memory.usage = (unsigned long long)tinfo.pti_resident_size;
                                 pt[i].cpu.time = (double)(tinfo.pti_total_user + tinfo.pti_total_system) / 100000000.; // The time is in nanoseconds, we store it as 1/10s
                                 pt[i].threads.self = tinfo.pti_threadnum;
                         }
@@ -210,8 +216,12 @@ int initprocesstree_sysdep(ProcessTree_T **reference, ProcessEngine_Flags pflags
                                         DEBUG("proc_pid_rusage for pid %d failed -- %s\n", pt[i].pid, STRERROR);
                         } else {
                                 pt[i].read.time = pt[i].write.time = Time_milli();
-                                pt[i].read.bytes = rusage.ri_diskio_bytesread;
-                                pt[i].write.bytes = rusage.ri_diskio_byteswritten;
+                                pt[i].read.bytes = -1;
+                                pt[i].read.bytesPhysical = rusage.ri_diskio_bytesread;
+                                pt[i].read.operations = -1;
+                                pt[i].write.bytes = -1;
+                                pt[i].write.bytesPhysical = rusage.ri_diskio_byteswritten;
+                                pt[i].write.operations = -1;
                         }
 #endif
                 }
@@ -244,7 +254,7 @@ int getloadavg_sysdep (double *loadv, int nelem) {
  * This routine returns real memory in use.
  * @return: true if successful, false if failed (or not available)
  */
-boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
+bool used_system_memory_sysdep(SystemInfo_T *si) {
         /* Memory */
         vm_statistics_data_t page_info;
         mach_msg_type_number_t count = HOST_VM_INFO_COUNT;
@@ -253,7 +263,7 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
                 DEBUG("system statistic error -- cannot get memory usage\n");
                 return false;
         }
-        si->memory.usage.bytes = (uint64_t)(page_info.wire_count + page_info.active_count) * (uint64_t)pagesize;
+        si->memory.usage.bytes = (unsigned long long)(page_info.wire_count + page_info.active_count) * (unsigned long long)pagesize;
 
         /* Swap */
         int mib[2] = {CTL_VM, VM_SWAPUSAGE};
@@ -264,8 +274,8 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
                 si->swap.size = 0ULL;
                 return false;
         }
-        si->swap.size = (uint64_t)swap.xsu_total;
-        si->swap.usage.bytes = (uint64_t)swap.xsu_used;
+        si->swap.size = (unsigned long long)swap.xsu_total;
+        si->swap.usage.bytes = (unsigned long long)swap.xsu_used;
 
         return true;
 }
@@ -275,7 +285,7 @@ boolean_t used_system_memory_sysdep(SystemInfo_T *si) {
  * This routine returns system/user CPU time in use.
  * @return: true if successful, false if failed
  */
-boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
+bool used_system_cpu_sysdep(SystemInfo_T *si) {
         long                      total;
         long                      total_new = 0;
         kern_return_t             kret;
@@ -291,13 +301,39 @@ boolean_t used_system_cpu_sysdep(SystemInfo_T *si) {
                 total_old = total_new;
 
                 si->cpu.usage.user = (total > 0) ? (100. * (double)(cpu_info.cpu_ticks[CPU_STATE_USER] - cpu_user_old) / total) : -1.;
+                si->cpu.usage.nice = (total > 0) ? (100. * (double)(cpu_info.cpu_ticks[CPU_STATE_NICE] - cpu_nice_old) / total) : -1.;
                 si->cpu.usage.system = (total > 0) ? (100. * (double)(cpu_info.cpu_ticks[CPU_STATE_SYSTEM] - cpu_syst_old) / total) : -1.;
-                si->cpu.usage.wait = 0.; /* there is no wait statistic available */
 
                 cpu_user_old = cpu_info.cpu_ticks[CPU_STATE_USER];
+                cpu_nice_old = cpu_info.cpu_ticks[CPU_STATE_NICE];
                 cpu_syst_old = cpu_info.cpu_ticks[CPU_STATE_SYSTEM];
 
                 return true;
         }
         return false;
 }
+
+
+bool used_system_filedescriptors_sysdep(__attribute__ ((unused)) SystemInfo_T *si) {
+        // Open files
+        size_t len = sizeof(si->filedescriptors.allocated);
+        if (sysctlbyname("kern.num_files", &si->filedescriptors.allocated, &len, NULL, 0) == -1) {
+                DEBUG("system statistics error -- sysctl kern.openfiles failed: %s\n", STRERROR);
+                return false;
+        }
+        // Max files
+        int mib[2] = {CTL_KERN, KERN_MAXFILES};
+        len = sizeof(si->filedescriptors.maximum);
+        if (sysctl(mib, 2, &si->filedescriptors.maximum, &len, NULL, 0) == -1) {
+                DEBUG("system statistics error -- sysctl kern.maxfiles failed: %s\n", STRERROR);
+                return false;
+        }
+        return true;
+}
+
+
+bool available_statistics(SystemInfo_T *si) {
+        si->statisticsAvailable = Statistics_CpuUser | Statistics_CpuSystem | Statistics_CpuNice | Statistics_FiledescriptorsPerSystem;
+        return true;
+}
+
