@@ -123,7 +123,7 @@
 
 #define T Ssl_T
 struct T {
-        boolean_t accepted;
+        bool accepted;
         int socket;
         SslOptions_T options;
         SSL *handler;
@@ -156,12 +156,12 @@ static Ssl_Version _optionsVersion(int version) {
 }
 
 
-static boolean_t _optionsVerify(short verify) {
+static bool _optionsVerify(short verify) {
         return verify != -1 ? verify : Run.ssl.verify != -1 ? Run.ssl.verify : false;
 }
 
 
-static boolean_t _optionsAllowSelfSigned(short allowSelfSigned) {
+static bool _optionsAllowSelfSigned(short allowSelfSigned) {
         return allowSelfSigned != -1 ? allowSelfSigned : Run.ssl.allowSelfSigned != -1 ? Run.ssl.allowSelfSigned : false;
 }
 
@@ -186,6 +186,16 @@ static const char *_optionsServerPEMFile(const char *pemfile) {
 }
 
 
+static const char *_optionsServerPEMChain(const char *pemchain) {
+        return pemchain ? pemchain : Run.ssl.pemchain ? Run.ssl.pemchain: NULL;
+}
+
+
+static const char *_optionsServerPEMKey(const char *pemkey) {
+        return pemkey ? pemkey : Run.ssl.pemkey ? Run.ssl.pemkey: NULL;
+}
+
+
 static const char *_optionsClientPEMFile(const char *clientpemfile) {
         return clientpemfile ? clientpemfile : Run.ssl.clientpemfile ? Run.ssl.clientpemfile: NULL;
 }
@@ -201,95 +211,98 @@ static Hash_Type _optionsChecksumType(Hash_Type checksumType) {
 }
 
 
-static boolean_t _setVersion(SSL_CTX *ctx, SslOptions_T options) {
-        long version = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1;
+static bool _setVersion(SSL_CTX *ctx, SslOptions_T options) {
+        unsigned long versionMask = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1;
 #if defined HAVE_TLSV1_1
-        version |= SSL_OP_NO_TLSv1_1;
+        versionMask |= SSL_OP_NO_TLSv1_1;
 #endif
 #if defined HAVE_TLSV1_2
-        version |= SSL_OP_NO_TLSv1_2;
+        versionMask |= SSL_OP_NO_TLSv1_2;
 #endif
 #if defined HAVE_TLSV1_3
-        version |= SSL_OP_NO_TLSv1_3;
+        versionMask |= SSL_OP_NO_TLSv1_3;
 #endif
-        switch (_optionsVersion(options->version)) {
-                case SSL_V2:
+
+        short versionAllowed = _optionsVersion(options->version);
+
+        if (versionAllowed & SSL_V2) {
 #if defined OPENSSL_NO_SSL2 || ! defined HAVE_SSLV2
-                        LogError("SSL: SSLv2 not supported\n");
-                        return false;
+                LogError("SSL: SSLv2 not supported\n");
+                return false;
 #else
-                        if (Run.flags & Run_FipsEnabled) {
-                                LogError("SSL: SSLv2 is not allowed in FIPS mode -- use TLS\n");
-                                return false;
-                        }
-                        version &= ~SSL_OP_NO_SSLv2;
-#endif
-                        break;
-                case SSL_V3:
-#if defined OPENSSL_NO_SSL3
-                        LogError("SSL: SSLv3 not supported\n");
+                if (Run.flags & Run_FipsEnabled) {
+                        LogError("SSL: SSLv2 is not allowed in FIPS mode -- use TLS\n");
                         return false;
-#else
-                        if (Run.flags & Run_FipsEnabled) {
-                                LogError("SSL: SSLv3 is not allowed in FIPS mode -- use TLS\n");
-                                return false;
-                        }
-                        version &= ~SSL_OP_NO_SSLv3;
+                }
+                versionMask &= ~SSL_OP_NO_SSLv2;
 #endif
-                        break;
-                case SSL_TLSV1:
-#if defined OPENSSL_NO_TLS1_METHOD
-                        LogError("SSL: TLSv1.0 not supported\n");
-                        return false;
-#else
-                        version &= ~SSL_OP_NO_TLSv1;
-#endif
-                        break;
-                case SSL_TLSV11:
-#if defined OPENSSL_NO_TLS1_1_METHOD || ! defined HAVE_TLSV1_1
-                        LogError("SSL: TLSv1.1 not supported\n");
-                        return false;
-#else
-                        version &= ~SSL_OP_NO_TLSv1_1;
-#endif
-                        break;
-                case SSL_TLSV12:
-#if defined OPENSSL_NO_TLS1_2_METHOD || ! defined HAVE_TLSV1_2
-                        LogError("SSL: TLSv1.2 not supported\n");
-                        return false;
-#else
-                        version &= ~SSL_OP_NO_TLSv1_2;
-#endif
-                        break;
-                case SSL_TLSV13:
-#if defined OPENSSL_NO_TLS1_3_METHOD || ! defined HAVE_TLSV1_3
-                        LogError("SSL: TLSv1.3 not supported\n");
-                        return false;
-#else
-                        version &= ~SSL_OP_NO_TLSv1_3;
-#endif
-                        break;
-                case SSL_Auto:
-                default:
-                        // Enable TLS protocols by default
-                        version &= ~SSL_OP_NO_TLSv1;
-#if defined HAVE_TLSV1_1
-                        version &= ~SSL_OP_NO_TLSv1_1;
-#endif
-#if defined HAVE_TLSV1_2
-                        version &= ~SSL_OP_NO_TLSv1_2;
-#endif
-#if defined HAVE_TLSV1_3
-                        version &= ~SSL_OP_NO_TLSv1_3;
-#endif
-                        break;
         }
-        SSL_CTX_set_options(ctx, version);
+
+        if (versionAllowed & SSL_V3) {
+#if defined OPENSSL_NO_SSL3
+                LogError("SSL: SSLv3 not supported\n");
+                return false;
+#else
+                if (Run.flags & Run_FipsEnabled) {
+                        LogError("SSL: SSLv3 is not allowed in FIPS mode -- use TLS\n");
+                        return false;
+                }
+                versionMask &= ~SSL_OP_NO_SSLv3;
+#endif
+        }
+
+        if (versionAllowed & SSL_TLSV1) {
+#if defined OPENSSL_NO_TLS1_METHOD
+                LogError("SSL: TLSv1.0 not supported\n");
+                return false;
+#else
+                versionMask &= ~SSL_OP_NO_TLSv1;
+#endif
+        }
+
+        if (versionAllowed & SSL_TLSV11) {
+#if defined OPENSSL_NO_TLS1_1_METHOD || ! defined HAVE_TLSV1_1
+                LogError("SSL: TLSv1.1 not supported\n");
+                return false;
+#else
+                versionMask &= ~SSL_OP_NO_TLSv1_1;
+#endif
+        }
+
+        if (versionAllowed & SSL_TLSV12) {
+#if defined OPENSSL_NO_TLS1_2_METHOD || ! defined HAVE_TLSV1_2
+                LogError("SSL: TLSv1.2 not supported\n");
+                return false;
+#else
+                versionMask &= ~SSL_OP_NO_TLSv1_2;
+#endif
+        }
+
+        if (versionAllowed & SSL_TLSV13) {
+#if defined OPENSSL_NO_TLS1_3_METHOD || ! defined HAVE_TLSV1_3
+                LogError("SSL: TLSv1.3 not supported\n");
+                return false;
+#else
+                versionMask &= ~SSL_OP_NO_TLSv1_3;
+#endif
+        }
+
+        if (versionAllowed == SSL_Auto) {
+                // Enable TLS 1.2 and 1.3 protocols by default
+#if ! defined OPENSSL_NO_TLS1_2_METHOD && defined HAVE_TLSV1_2
+                versionMask &= ~SSL_OP_NO_TLSv1_2;
+#endif
+#if ! defined OPENSSL_NO_TLS1_3_METHOD && defined HAVE_TLSV1_3
+                versionMask &= ~SSL_OP_NO_TLSv1_3;
+#endif
+        }
+
+        SSL_CTX_set_options(ctx, versionMask);
         return true;
 }
 
 
-static boolean_t _retry(int socket, int *timeout, int (*callback)(int socket, time_t milliseconds)) {
+static bool _retry(int socket, int *timeout, bool (*callback)(int socket, time_t milliseconds)) {
         long long start = Time_milli();
         if (callback(socket, *timeout)) {
                 long long stop = Time_milli();
@@ -307,7 +320,7 @@ static void _threadID(CRYPTO_THREADID *id) {
 }
 
 
-static void _mutexLock(int mode, int n, const char *file, int line) {
+static void _mutexLock(int mode, int n, __attribute__ ((unused)) const char *file, __attribute__ ((unused)) int line) {
         if (mode & CRYPTO_LOCK)
                 Mutex_lock(instanceMutexTable[n]);
         else
@@ -437,7 +450,7 @@ static int _verifyClientCertificates(int preverify_ok, X509_STORE_CTX *ctx) {
 }
 
 
-static boolean_t _setServerNameIdentification(T C, const char *hostname) {
+static bool _setServerNameIdentification(T C, const char *hostname) {
 #ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
         struct sockaddr_storage addr;
         // If the name is set and we use TLS protocol, enable the SNI extension (provided the hostname value is not an IP address)
@@ -454,7 +467,7 @@ static boolean_t _setServerNameIdentification(T C, const char *hostname) {
 }
 
 
-static boolean_t _setClientCertificate(T C, const char *file) {
+static bool _setClientCertificate(T C, const char *file) {
         if (SSL_CTX_use_certificate_chain_file(C->ctx, file) != 1) {
                 LogError("SSL client certificate chain loading failed: %s\n", SSLERROR);
                 return false;
@@ -515,14 +528,14 @@ void Ssl_threadCleanup() {
 }
 
 
-void Ssl_setFipsMode(boolean_t enabled) {
 #ifdef OPENSSL_FIPS
+void Ssl_setFipsMode(bool enabled) {
         if (enabled && ! FIPS_mode() && ! FIPS_mode_set(1))
                 THROW(AssertException, "SSL: cannot enter FIPS mode -- %s", SSLERROR);
         else if (! enabled && FIPS_mode() && ! FIPS_mode_set(0))
                 THROW(AssertException, "SSL: cannot exit FIPS mode -- %s", SSLERROR);
-#endif
 }
+#endif
 
 
 T Ssl_new(SslOptions_T options) {
@@ -592,7 +605,7 @@ void Ssl_free(T *C) {
 
 void Ssl_close(T C) {
         ASSERT(C);
-        boolean_t retry = false;
+        bool retry = false;
         int timeout = Run.limits.networkTimeout;
         do {
                 int rv = SSL_shutdown(C->handler);
@@ -629,7 +642,7 @@ void Ssl_connect(T C, int socket, int timeout, const char *name) {
         SSL_set_connect_state(C->handler);
         SSL_set_fd(C->handler, C->socket);
         _setServerNameIdentification(C, name);
-        boolean_t retry = false;
+        bool retry = false;
         do {
                 int rv = SSL_connect(C->handler);
                 if (rv < 0) {
@@ -657,11 +670,11 @@ void Ssl_connect(T C, int socket, int timeout, const char *name) {
 }
 
 
-int Ssl_write(T C, void *b, int size, int timeout) {
+int Ssl_write(T C, const void *b, int size, int timeout) {
         ASSERT(C);
         int n = 0;
         if (size > 0) {
-                boolean_t retry = false;
+                bool retry = false;
                 do {
                         switch (SSL_get_error(C->handler, (n = SSL_write(C->handler, b, size)))) {
                                 case SSL_ERROR_NONE:
@@ -702,7 +715,7 @@ int Ssl_read(T C, void *b, int size, int timeout) {
         ASSERT(C);
         int n = 0;
         if (size > 0) {
-                boolean_t retry = false;
+                bool retry = false;
                 do {
                         switch (SSL_get_error(C->handler, (n = SSL_read(C->handler, b, size)))) {
                                 case SSL_ERROR_NONE:
@@ -741,14 +754,14 @@ int Ssl_read(T C, void *b, int size, int timeout) {
 
 int Ssl_getCertificateValidDays(T C) {
         if (C && C->certificate) {
-                // Certificates which expired already are catched in preverify => we don't need to handle them here
-                int deltadays = 0;
+                // Certificates which expired already are caught in preverify => we don't need to handle them here
+                volatile int deltadays = 0;
 #ifdef HAVE_ASN1_TIME_DIFF
                 int deltaseconds;
 #if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
-                if (! ASN1_TIME_diff(&deltadays, &deltaseconds, NULL, X509_get_notAfter(C->certificate))) {
+                if (! ASN1_TIME_diff((int *)&deltadays, &deltaseconds, NULL, X509_get_notAfter(C->certificate))) {
 #else
-                if (! ASN1_TIME_diff(&deltadays, &deltaseconds, NULL, X509_get0_notAfter(C->certificate))) {
+                if (! ASN1_TIME_diff((int *)&deltadays, &deltaseconds, NULL, X509_get0_notAfter(C->certificate))) {
 #endif
                         THROW(IOException, "invalid time format in certificate's notAfter field");
                 }
@@ -784,7 +797,20 @@ char *Ssl_printOptions(SslOptions_T options, char *b, int size) {
         if (options->flags) {
                 int count = 0;
                 if (options->version != -1) {
-                        snprintf(b + strlen(b), size - strlen(b) - 1, "version: %s", sslnames[options->version]);
+                        int versions = 0;
+                        snprintf(b + strlen(b), size - strlen(b) - 1, "version: ");
+                        if (options->version & SSL_V2)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sSSLv2", versions++ ? " " : "");
+                        if (options->version & SSL_V3)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sSSLv3", versions++ ? " " : "");
+                        if (options->version & SSL_TLSV1)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sTLSv1.0", versions++ ? " " : "");
+                        if (options->version & SSL_TLSV11)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sTLSv1.1", versions++ ? " " : "");
+                        if (options->version & SSL_TLSV12)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sTLSv1.2", versions++ ? " " : "");
+                        if (options->version & SSL_TLSV13)
+                                snprintf(b + strlen(b), size - strlen(b) - 1, "%sTLSv1.3", versions++ ? " " : "");
                         count++;
                 }
                 if (options->verify == true)
@@ -793,6 +819,10 @@ char *Ssl_printOptions(SslOptions_T options, char *b, int size) {
                         snprintf(b + strlen(b), size - strlen(b) - 1, "%sselfsigned: allow", count++ ? ", " : "");
                 if (options->pemfile)
                         snprintf(b + strlen(b), size - strlen(b) - 1, "%spemfile: %s", count ++ ? ", " : "", options->pemfile);
+                if (options->pemchain)
+                        snprintf(b + strlen(b), size - strlen(b) - 1, "%spemchain: %s", count ++ ? ", " : "", options->pemchain);
+                if (options->pemkey)
+                        snprintf(b + strlen(b), size - strlen(b) - 1, "%spemkey: %s", count ++ ? ", " : "", options->pemkey);
                 if (options->clientpemfile)
                         snprintf(b + strlen(b), size - strlen(b) - 1, "%sclientpemfile: %s", count ++ ? ", " : "", options->clientpemfile);
                 if (options->CACertificateFile)
@@ -864,12 +894,14 @@ SslServer_T SslServer_new(int socket, SslOptions_T options) {
         SSL_CTX_set_options(S->ctx, SSL_OP_NO_COMPRESSION);
 #endif
         SSL_CTX_set_session_cache_mode(S->ctx, SSL_SESS_CACHE_OFF);
+        const char *pemchain = _optionsServerPEMChain(options->pemchain);
+        const char *pemkey = _optionsServerPEMKey(options->pemkey);
         const char *pemfile = _optionsServerPEMFile(options->pemfile);
-        if (SSL_CTX_use_certificate_chain_file(S->ctx, pemfile) != 1) {
+        if (SSL_CTX_use_certificate_chain_file(S->ctx, pemchain ? pemchain : pemfile) != 1) {
                 LogError("SSL: server certificate chain loading failed -- %s\n", SSLERROR);
                 goto sslerror;
         }
-        if (SSL_CTX_use_PrivateKey_file(S->ctx, pemfile, SSL_FILETYPE_PEM) != 1) {
+        if (SSL_CTX_use_PrivateKey_file(S->ctx, pemkey ? pemkey : pemfile, SSL_FILETYPE_PEM) != 1) {
                 LogError("SSL: server private key loading failed -- %s\n", SSLERROR);
                 goto sslerror;
         }
@@ -941,13 +973,13 @@ void SslServer_freeConnection(SslServer_T S, T *C) {
 }
 
 
-boolean_t SslServer_accept(T C, int socket, int timeout) {
+bool SslServer_accept(T C, int socket, int timeout) {
         ASSERT(C);
         ASSERT(socket >= 0);
         C->socket = socket;
         SSL_set_accept_state(C->handler);
         SSL_set_fd(C->handler, C->socket);
-        boolean_t retry = false;
+        bool retry = false;
         do {
                 int rv = SSL_accept(C->handler);
                 if (rv < 0) {
