@@ -120,27 +120,27 @@ typedef struct mystate4 {
         int32_t            ncycle;
         union {
                 struct {
-                        uint64_t atime;
-                        uint64_t ctime;
-                        uint64_t mtime;
+                        unsigned long long atime;
+                        unsigned long long ctime;
+                        unsigned long long mtime;
                         int32_t mode;
                 } directory;
 
                 struct {
-                        uint64_t inode;
-                        uint64_t readpos;
-                        uint64_t size;
-                        uint64_t atime;
-                        uint64_t ctime;
-                        uint64_t mtime;
+                        unsigned long long inode;
+                        unsigned long long readpos;
+                        unsigned long long size;
+                        unsigned long long atime;
+                        unsigned long long ctime;
+                        unsigned long long mtime;
                         int32_t mode;
                         MD_T hash;
                 } file;
 
                 struct {
-                        uint64_t atime;
-                        uint64_t ctime;
-                        uint64_t mtime;
+                        unsigned long long atime;
+                        unsigned long long ctime;
+                        unsigned long long mtime;
                         int32_t mode;
                 } fifo;
 
@@ -150,7 +150,7 @@ typedef struct mystate4 {
 
                 struct {
                         int32_t duplex;
-                        int64_t speed;
+                        long long speed;
                         //FIXME: when Link API is moved from libmonit to monit, save also link bytes in/out and packets in/out history, so the network statistics is not reset on each monit reload
                 } net;
         } priv;
@@ -166,21 +166,21 @@ typedef struct mystate3 {
         int32_t            ncycle;
         union {
                 struct {
-                        uint64_t timestamp;
+                        unsigned long long timestamp;
                         int32_t mode;
                 } directory;
 
                 struct {
-                        uint64_t inode;
-                        uint64_t readpos;
-                        uint64_t size;
-                        uint64_t timestamp;
+                        unsigned long long inode;
+                        unsigned long long readpos;
+                        unsigned long long size;
+                        unsigned long long timestamp;
                         int32_t mode;
                         MD_T hash;
                 } file;
 
                 struct {
-                        uint64_t timestamp;
+                        unsigned long long timestamp;
                         int32_t mode;
                 } fifo;
 
@@ -191,7 +191,7 @@ typedef struct mystate3 {
 
                 struct {
                         int32_t duplex;
-                        int64_t speed;
+                        long long speed;
                 } net;
         } priv;
 } State3_T;
@@ -210,8 +210,8 @@ typedef struct mystate1 {
         int32_t            ncycle;
         union {
                 struct {
-                        uint64_t inode;
-                        uint64_t readpos;
+                        unsigned long long inode;
+                        unsigned long long readpos;
                 } file;
         } priv;
 } State1_T;
@@ -224,13 +224,13 @@ typedef struct mystate0 {
         int32_t  nstart;
         int32_t  ncycle;
         int32_t  monitor;
-        uint64_t error;               // obsolete since Monit 5.0
+        unsigned long long error;               // obsolete since Monit 5.0
 } State0_T;
 
 
 static int file = -1;
-static uint64_t booted = 0ULL;
-static boolean_t _stateDirty = false;
+static unsigned long long booted = 0ULL;
+static bool _stateDirty = false;
 
 
 /* ----------------------------------------------------------------- Private */
@@ -256,16 +256,17 @@ static void _updateMonitor(Service_T S, Monitor_State monitor) {
                 else
                         S->monitor = Monitor_Init;
         }
+        S->onrebootRestored = true;
 }
 
 
-static void _updateFilePosition(Service_T S, uint64_t inode, uint64_t readpos) {
+static void _updateFilePosition(Service_T S, unsigned long long inode, unsigned long long readpos) {
         S->inf.file->inode = (ino_t)inode;
         S->inf.file->readpos = (off_t)readpos;
 }
 
 
-static void _updateTimestamp(Service_T S, uint64_t atime, uint64_t ctime, uint64_t mtime) {
+static void _updateTimestamp(Service_T S, unsigned long long atime, unsigned long long ctime, unsigned long long mtime) {
         for (Timestamp_T t = S->timestamplist; t; t = t->next) {
                 if (t->test_changes) {
                         switch (t->type) {
@@ -294,7 +295,7 @@ static void _updatePermission(Service_T S, int mode) {
 }
 
 
-static void _updateSize(Service_T S, int64_t size) {
+static void _updateSize(Service_T S, long long size) {
         for (Size_T s = S->sizelist; s; s = s->next) {
                 if (s->test_changes) {
                         s->size = size;
@@ -312,7 +313,7 @@ static void _updateChecksum(Service_T S, char *hash) {
 }
 
 
-static void _updateLinkSpeed(Service_T S, int32_t duplex, int64_t speed) {
+static void _updateLinkSpeed(Service_T S, int32_t duplex, long long speed) {
         for (LinkSpeed_T l = S->linkspeedlist; l; l = l->next) {
                 l->duplex = duplex;
                 l->speed = speed;
@@ -320,7 +321,7 @@ static void _updateLinkSpeed(Service_T S, int32_t duplex, int64_t speed) {
 }
 
 
-static void _restoreV4() {
+static void _restoreV4(void) {
         // System header
         if (read(file, &booted, sizeof(booted)) != sizeof(booted)) {
                 THROW(IOException, "Unable to read system boot time");
@@ -367,7 +368,7 @@ static void _restoreV4() {
 }
 
 
-static void _restoreV3() {
+static void _restoreV3(void) {
         // System header
         if (read(file, &booted, sizeof(booted)) != sizeof(booted)) {
                 THROW(IOException, "Unable to read system boot time");
@@ -411,7 +412,7 @@ static void _restoreV3() {
 }
 
 
-static void _restoreV2() {
+static void _restoreV2(void) {
         // System header
         booted = systeminfo.booted; // No boot time available => for backward compatibility, act as if the system was not rebooted, as we don't know if monit was only restarted or machine rebooted
         // Services state
@@ -453,7 +454,7 @@ static void _restoreV2() {
 }
 
 
-static void _restoreV1() {
+static void _restoreV1(void) {
         // System header
         booted = systeminfo.booted; // No boot time available => for backward compatibility, act as if the system was not rebooted, as we don't know if monit was only restarted or machine rebooted
         // Services state
@@ -490,7 +491,7 @@ static void _restoreV0(int services) {
 /* ------------------------------------------------------------------ Public */
 
 
-boolean_t State_open() {
+bool State_open() {
         State_close();
         if ((file = open(Run.files.state, O_RDWR | O_CREAT, 0600)) == -1) {
                 LogError("State file '%s': cannot open for write -- %s\n", Run.files.state, STRERROR);
@@ -542,28 +543,28 @@ void State_save() {
                         state.ncycle = service->ncycle;
                         switch (service->type) {
                                 case Service_Directory:
-                                        state.priv.directory.atime = (uint64_t)service->inf.directory->timestamp.access;
-                                        state.priv.directory.ctime = (uint64_t)service->inf.directory->timestamp.change;
-                                        state.priv.directory.mtime = (uint64_t)service->inf.directory->timestamp.modify;
+                                        state.priv.directory.atime = (unsigned long long)service->inf.directory->timestamp.access;
+                                        state.priv.directory.ctime = (unsigned long long)service->inf.directory->timestamp.change;
+                                        state.priv.directory.mtime = (unsigned long long)service->inf.directory->timestamp.modify;
                                         state.priv.directory.mode = service->inf.directory->mode;
                                         break;
 
                                 case Service_Fifo:
-                                        state.priv.fifo.atime = (uint64_t)service->inf.fifo->timestamp.access;
-                                        state.priv.fifo.ctime = (uint64_t)service->inf.fifo->timestamp.change;
-                                        state.priv.fifo.mtime = (uint64_t)service->inf.fifo->timestamp.modify;
+                                        state.priv.fifo.atime = (unsigned long long)service->inf.fifo->timestamp.access;
+                                        state.priv.fifo.ctime = (unsigned long long)service->inf.fifo->timestamp.change;
+                                        state.priv.fifo.mtime = (unsigned long long)service->inf.fifo->timestamp.modify;
                                         state.priv.fifo.mode = service->inf.fifo->mode;
                                         break;
 
                                 case Service_File:
                                         state.priv.file.inode = service->inf.file->inode;
                                         state.priv.file.readpos = service->inf.file->readpos;
-                                        state.priv.file.size = (int64_t)service->inf.file->size;
-                                        state.priv.file.atime = (uint64_t)service->inf.file->timestamp.access;
-                                        state.priv.file.ctime = (uint64_t)service->inf.file->timestamp.change;
-                                        state.priv.file.mtime = (uint64_t)service->inf.file->timestamp.modify;
+                                        state.priv.file.size = (long long)service->inf.file->size;
+                                        state.priv.file.atime = (unsigned long long)service->inf.file->timestamp.access;
+                                        state.priv.file.ctime = (unsigned long long)service->inf.file->timestamp.change;
+                                        state.priv.file.mtime = (unsigned long long)service->inf.file->timestamp.modify;
                                         state.priv.file.mode = service->inf.file->mode;
-                                        strncpy(state.priv.file.hash, service->inf.file->cs_sum, sizeof(state.priv.file.hash) - 1);
+                                        snprintf(state.priv.file.hash, sizeof(state.priv.file.hash), "%s", service->inf.file->cs_sum);
                                         break;
 
                                 case Service_Filesystem:
@@ -662,7 +663,7 @@ void State_restore() {
 }
 
 
-boolean_t State_reboot() {
+bool State_reboot() {
         return systeminfo.booted == booted ? false : true;
 }
 
